# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import CatalogAPI as CatalogAPI
import CatalogAPI.Catalog
import LiveRenderOpChain as LiveRenderOpChain
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI as Nodes3DAPI
import PyFnAttribute
import PyFnGeolib
import RenderManager.RenderSettings
import Utils as Utils
from Callbacks.Callbacks import Callbacks as Callbacks
from Nodes3DAPI.PortOpClient import GraphStateSpec as GraphStateSpec, PortOpClient as PortOpClient
from PyUtilModule.RenderManager.Constants import RenderModes as RenderModes
from PyUtilModule.WorkingSetManager import WorkingSetManager as WorkingSetManager
from builtins import OpTreeCommandRequestQueue, OpTreeCommandRequestTransactionAdaptor
from typing import Any, ClassVar, Set, Tuple

class LiveRenderPortOpObserver(PortOpClient):
    _abc_impl: ClassVar[_abc_data] = ...
    _kDynamicGSV: ClassVar[str] = ...
    _kRenderCancelledEvent: ClassVar[str] = ...
    _kRenderFinalizedEvent: ClassVar[str] = ...
    _kStaticGSV: ClassVar[str] = ...
    _kTerminalOpDirtyEvents: ClassVar[tuple] = ...
    _kUnregisterEventName: ClassVar[str] = ...
    _kVirtualCameraMoveEvent: ClassVar[str] = ...
    _observers: ClassVar[dict] = ...
    __abstractmethods__: ClassVar[frozenset] = ...
    def __init__(self, runtime: PyFnGeolib.GeolibRuntime, node: NodegraphAPI.Node, graphState: NodegraphAPI.GraphState, mainSequenceID: int, opChain: LiveRenderOpChain, liveAttributeQueue: LiveRenderOpChain.LiveAttributeQueue, renderingSettings: RenderManager.RenderSettings.RenderingSettings, projectSnapshot: ProjectSnapshot | None) -> None: ...
    def _LiveRenderPortOpObserver__commitGSVToGraphStateSpec(self, gsvID: str): ...
    @staticmethod
    def _LiveRenderPortOpObserver__copyGraphStateEntry(sourceGraphState: NodegraphAPI.GraphState, targetGraphStateBuilder: NodegraphAPI.GraphStateBuilder, variableName: str): ...
    @staticmethod
    def _LiveRenderPortOpObserver__encodeGSVNames(graphState: NodegraphAPI.GraphState) -> set[str]: ...
    @classmethod
    def _LiveRenderPortOpObserver__getGeolibServiceHostAndPort(cls, mainSequenceID): ...
    @classmethod
    def _LiveRenderPortOpObserver__onCatalogGSVPinned(cls, objectHash: object, catalogItem: CatalogAPI.Catalog.CatalogItem, gsvID: str, isPinned: bool): ...
    @classmethod
    def _LiveRenderPortOpObserver__onDirtyEvent(cls, args: tuple, kwargs: dict): ...
    def _LiveRenderPortOpObserver__onNodeDelete(self, eventType: str, eventID: int, **kwargs): ...
    @classmethod
    def _LiveRenderPortOpObserver__onRenderCancelled(cls, eventType: str, mainSequenceIDs: tuple[int, ...]): ...
    @classmethod
    def _LiveRenderPortOpObserver__onRenderFinalized(cls, eventType: str, eventID: int, renderMethod: str, runtime: PyFnGeolib.GeolibRuntime, node: NodegraphAPI.Node, graphState: NodegraphAPI.GraphState, renderUUID: str, renderer: str, renderingSettings: RenderManager.RenderSettings.RenderingSettings, projectSnapshot: ProjectSnapshot | None): ...
    @classmethod
    def _LiveRenderPortOpObserver__onUnregister(cls, eventType: str, eventID: int): ...
    @classmethod
    def _LiveRenderPortOpObserver__onVirtualCameraMoved(cls, eventType: str, eventID: object, attrName: str, attrValue: PyFnAttribute.Attribute, isFinal: bool): ...
    @staticmethod
    def _LiveRenderPortOpObserver__tryRegisterEventHandler(eventType, fn): ...
    @staticmethod
    def _LiveRenderPortOpObserver__tryUnregisterEventHandler(eventType, fn): ...
    def _LiveRenderPortOpObserver__unregister(self): ...
    def _LiveRenderPortOpObserver__updateCatalogItem(self, graphState: NodegraphAPI.GraphState): ...
    def addImmutableGraphStateVariable(self, variableName: str): ...
    def getImmutableGraphStateVariables(self) -> set[str]: ...
    def getOp(self, node: NodegraphAPI.Node, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, visitedState: set, txn: PyFnGeolib.GeolibRuntimeTransaction) -> PyFnGeolib.GeolibRuntimeOp: ...
    def getTxnQueue(self) -> OpTreeCommandRequestQueue: ...
    def modifyInitialGraphState(self, graphState: NodegraphAPI.GraphState) -> NodegraphAPI.GraphState: ...
    def opChanged(self, op: PyFnGeolib.GeolibRuntimeOp, graphState: NodegraphAPI.GraphState, txn: PyFnGeolib.GeolibRuntimeTransaction): ...
    @classmethod
    def queueLiveAttribute(cls, locationPath: str, attrPath: str, attrValue: str, attrEditor: PyFnAttribute.GroupAttribute | None): ...
    @classmethod
    def registerEvents(cls): ...
    @classmethod
    def registerLiveRender(cls, runtime: PyFnGeolib.GeolibRuntime, node: NodegraphAPI.Node, graphState: NodegraphAPI.GraphState, mainSequenceID: int, renderUUID: str, renderingSettings: RenderManager.RenderSettings.RenderingSettings, projectSnapshot: ProjectSnapshot | None) -> LiveRenderPortOpObserver: ...
    def removeImmutableGraphStateVariable(self, variableName: str): ...
    @classmethod
    def unregisterEvents(cls): ...
    @classmethod
    def unregisterLiveRender(cls, mainSequenceID: int): ...
    @property
    def mainSequenceID(self): ...

class _RemoteGeolibUnavailableError(Exception): ...

class _RemoteRenderSpoolingError(Exception): ...

class _TransactionProxy:
    def __init__(self, txn: PyFnGeolib.GeolibRuntimeTransaction, txnAdaptor: OpTreeCommandRequestTransactionAdaptor) -> None: ...
    def createOp(self) -> PyFnGeolib.GeolibRuntimeOp: ...
    def localOnly(self) -> None: ...
    def setClientOp(self, client: PyFnGeolib.GeolibRuntimeClient, op: PyFnGeolib.GeolibRuntimeOp): ...
    def setNodeName(self, op: PyFnGeolib.GeolibRuntimeOp, name: str): ...
    def setNodeType(self, op: PyFnGeolib.GeolibRuntimeOp, nodeType: str): ...
    def setOpArgs(self, op: PyFnGeolib.GeolibRuntimeOp, opType: str, args: PyFnAttribute.GroupAttribute): ...
    def setOpInputs(self, op: PyFnGeolib.GeolibRuntimeOp, inputOps: list[PyFnGeolib.GeolibRuntimeOp]): ...
    def setTag(self, op: PyFnGeolib.GeolibRuntimeOp, tag: str): ...
    def useOpPool(self, opPool: LiveRenderOpChain._OpPool) -> None: ...
    def __getattr__(self, item: str) -> Any: ...
    @property
    def localTransaction(self): ...
    @property
    def remoteTransaction(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import typing
from . import Util as Util
from collections.abc import abstractmethod
from typing import Any, Generic, Set, Tuple, TypeVar, overload
T = TypeVar("T")

GroupBuilderBuildAndFlush: int
GroupBuilderBuildAndRetain: int
GroupBuilderModeNormal: int
GroupBuilderModeStrict: int

class Attribute:
    def __init__(self) -> None: ...
    def getBinary(self) -> bytes: ...
    def getHash(self) -> str: ...
    def getHash64(self) -> int: ...
    def getSize(self) -> int: ...
    def getXML(self) -> str: ...
    @classmethod
    def parseBinary(cls, binary: object) -> Attribute: ...
    @classmethod
    def parseXML(cls, xml: str) -> Attribute: ...
    def writePython(self, stream: object) -> Any: ...
    @overload
    def __eq__(self, other: object) -> bool: ...
    @overload
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    @overload
    def __ne__(self, other: object) -> bool: ...
    @overload
    def __ne__(self, other: object) -> bool: ...

class ConstVector(Generic[T]):
    def __init__(self) -> None: ...
    def count(self, object: object) -> int: ...
    def index(self, object: object) -> int: ...
    def toFnAttr(self, *args, **kwargs): ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    @overload
    def __getitem__(self, index: int) -> T: ...
    @overload
    def __getitem__(self, index: slice) -> ConstVector[T]: ...
    def __iter__(self) -> typing.Iterator[T]: ...
    def __gt__(self, other: object) -> bool: ...
    def __iter__(self) -> typing.Iterator[T]: ...
    def __le__(self, other: object) -> bool: ...
    def __len__(self) -> int: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

class DataAttribute(Attribute, Generic[T]):
    def __init__(self) -> None: ...
    def getBoundingSampleTimes(self, time: float) -> tuple: ...
    @abstractmethod
    def getData(self) -> ConstVector[T]: ...
    @abstractmethod
    def getNearestSample(self, sampleTime: float) -> ConstVector[T]: ...
    def getNumberOfTimeSamples(self) -> int: ...
    def getNumberOfTuples(self) -> int: ...
    def getNumberOfValues(self) -> int: ...
    def getSampleTime(self, index: int) -> float: ...
    def getSampleTimes(self) -> tuple: ...
    @abstractmethod
    def getSamples(self) -> dict[float, ConstVector[T]]: ...
    def getTupleSize(self) -> int: ...
    @abstractmethod
    def getValue(self, defaultValue: T = ..., throwOnError: bool = ...) -> T: ...

class DoubleAttribute(DataAttribute[float]):
    @overload
    def __init__(self, value: float, tupleSize: int = ...) -> None: ...
    @overload
    def __init__(self, data: dict, tupleSize: int = ...) -> None: ...
    @overload
    def __init__(self, data: typing.Sequence, tupleSize: int = ...) -> None: ...
    def getData(self) -> ConstVector[float]: ...
    def getNearestSample(self, sampleTime: float) -> ConstVector[float]: ...
    def getSamples(self) -> dict[float, ConstVector[float]]: ...
    def getValue(self, defaultValue: float = ..., throwOnError: bool = ...) -> float: ...

class FloatAttribute(DataAttribute[float]):
    @overload
    def __init__(self, value: float, tupleSize: int = ...) -> None: ...
    @overload
    def __init__(self, data: dict, tupleSize: int = ...) -> None: ...
    @overload
    def __init__(self, data: typing.Sequence, tupleSize: int = ...) -> None: ...
    def getData(self) -> ConstVector[float]: ...
    def getNearestSample(self, sampleTime: float) -> ConstVector[float]: ...
    def getSamples(self) -> dict[float, ConstVector[float]]: ...
    def getValue(self, defaultValue: float = ..., throwOnError: bool = ...) -> float: ...

class GroupAttribute(Attribute):
    @overload
    def __init__(self, groupInherit: bool = ...) -> None: ...
    @overload
    def __init__(self, children: typing.Sequence, groupInherit: bool = ...) -> None: ...
    @overload
    def __init__(self, children: dict, groupInherit: bool = ...) -> None: ...
    def childList(self) -> list[tuple[str, Attribute]]: ...
    def getChildByIndex(self, index: int) -> Attribute: ...
    def getChildByName(self, name: str) -> Attribute: ...
    def getChildName(self, index: int) -> str: ...
    def getGroupInherit(self) -> bool: ...
    def getNumberOfChildren(self) -> int: ...
    def writePython(self, stream: object) -> Any: ...

class GroupBuilder:
    def __init__(self, builderMode: int = ...) -> None: ...
    def build(self, builderMode: int = ...) -> GroupAttribute: ...
    def deepUpdate(self, attr: GroupAttribute) -> GroupBuilder: ...
    def delete(self, pathStr: str) -> GroupBuilder: ...
    def reserve(self, count: int) -> GroupBuilder: ...
    @overload
    def set(self, pathStr: str, attr: Attribute, groupInherit: bool = ...) -> GroupBuilder: ...
    @overload
    def set(self, pathStr: str, value: int, groupInherit: bool = ...) -> GroupBuilder: ...
    @overload
    def set(self, pathStr: str, value: float, groupInherit: bool = ...) -> GroupBuilder: ...
    @overload
    def set(self, pathStr: str, value: float, groupInherit: bool = ...) -> GroupBuilder: ...
    @overload
    def set(self, pathStr: str, value: str, groupInherit: bool = ...) -> GroupBuilder: ...
    def setGroupInherit(self, groupInherit: bool) -> GroupBuilder: ...
    @overload
    def setWithUniqueName(self, pathStr: str, attr: Attribute, groupInherit: bool = ...) -> GroupBuilder: ...
    @overload
    def setWithUniqueName(self, pathStr: str, value: int, groupInherit: bool = ...) -> GroupBuilder: ...
    @overload
    def setWithUniqueName(self, pathStr: str, value: float, groupInherit: bool = ...) -> GroupBuilder: ...
    @overload
    def setWithUniqueName(self, pathStr: str, value: float, groupInherit: bool = ...) -> GroupBuilder: ...
    @overload
    def setWithUniqueName(self, pathStr: str, value: str, groupInherit: bool = ...) -> GroupBuilder: ...
    def sort(self) -> GroupBuilder: ...
    def update(self, attr: GroupAttribute) -> GroupBuilder: ...

class IntAttribute(DataAttribute[int]):
    @overload
    def __init__(self, value: float, tupleSize: int = ...) -> None: ...
    @overload
    def __init__(self, data: dict, tupleSize: int = ...) -> None: ...
    @overload
    def __init__(self, data: typing.Sequence, tupleSize: int = ...) -> None: ...
    def getData(self) -> ConstVector[int]: ...
    def getNearestSample(self, sampleTime: float) -> ConstVector[int]: ...
    def getSamples(self) -> dict[float, ConstVector[int]]: ...
    def getValue(self, defaultValue: int = ..., throwOnError: bool = ...) -> int: ...

class NullAttribute(Attribute):
    def __init__(self) -> None: ...
    def writePython(self, stream: object) -> Any: ...

class StringAttribute(DataAttribute[str]):
    @overload
    def __init__(self, value: str, tupleSize: int = ...) -> None: ...
    @overload
    def __init__(self, data: dict, tupleSize: int = ...) -> None: ...
    @overload
    def __init__(self, data: typing.Sequence, tupleSize: int = ...) -> None: ...
    def getData(self) -> ConstVector[str]: ...
    def getNearestSample(self, sampleTime: float) -> ConstVector[str]: ...
    def getSamples(self) -> dict[float, ConstVector[str]]: ...
    def getValue(self, defaultValue: str = ..., throwOnError: bool = ...) -> str: ...
    @overload
    def __eq__(self, other: object) -> bool: ...
    @overload
    def __eq__(self, other: object) -> bool: ...
    @overload
    def __eq__(self, other: object) -> bool: ...
    @overload
    def __ne__(self, other: object) -> bool: ...
    @overload
    def __ne__(self, other: object) -> bool: ...
    @overload
    def __ne__(self, other: object) -> bool: ...

def DelimiterDecode(encodedNameOrLocationPath: str) -> str: ...
def DelimiterEncode(nameOrLocationPath: str) -> str: ...
def GetTotalSize() -> int: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from typing import ClassVar, Set, Tuple

class OutputFormat:
    AS_ARCHIVE: ClassVar[str] = ...
    AS_DIRECTORY: ClassVar[str] = ...
    DEFAULT: ClassVar[str] = ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyUtilModule as PyUtilModule
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import KatanaResources as ResourceFiles
import Utils as Utils
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtWidgets
import QT4FormWidgets as QT4FormWidgets
import QT4FormWidgets.ValuePolicy
import PyQt5.QtCore as QtCore
import UI4 as UI4
from QT4FormWidgets.BaseValueFormWidget import BaseValueFormWidget
from QT4FormWidgets.WidgetFactory import WidgetFactory
from typing import ClassVar, Set, Tuple

class KeyboardShortcutFormWidget(BaseValueFormWidget):
    shortcutChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent: PyQt5.QtWidgets.QWidget, policy: QT4FormWidgets.ValuePolicy.AbstractValuePolicy, factory: WidgetFactory) -> None: ...
    def _KeyboardShortcutFormWidget__on_keyboardShortcutLineEdit_textChanged(self, text): ...
    def _buildControlWidget(self, layout: PyQt5.QtWidgets.QLayout) -> PyQt5.QtWidgets.QWidget: ...
    def _updateControlWidget(self): ...
    def setReadOnly(self, readOnly: bool): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import NodegraphAPI
import Nodes3DAPI.TransformUtil as TransformUtil
import typing
from Nodes3DAPI.Node3D import Node3D as Node3D
from _typeshed import Incomplete
from typing import Set, Tuple

class AbstractTransform(Node3D):
    def _getXformAttribute(self, frameTime, includeInteractive: bool = ..., graphState: typing.Optional[NodegraphAPI.GraphState] = ...): ...
    def canOverride(self, attrName): ...
    def findOverrideParameter(self, path, attrName, time, index: Incomplete | None = ..., editable: bool = ...): ...
    def setInteractiveTransform(self, path, absScale, absRotate, absTranslate, time): ...
    def setInteractiveTransformFlag(self, path, time): ...
    def setOverride(self, path, attrName, time, attrData, index: Incomplete | None = ..., *args, **kwargs): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import AssetAPI as AssetAPI
import AssetBrowser as AssetBrowser
import PyUtilModule.AttrDump as AttrDump
import CacheManager as CacheManager
import CatalogAPI as CatalogAPI
import PyUtilModule.ChildProcess as ChildProcess
import PyUtilModule.ColorPaletteManager as ColorPaletteManager
import ConfigurationAPI_cmodule as Configuration
import PyUtilModule.Decorators as Decorators
import PyUtilModule.Documentation as Documentation
import DrawingModule as DrawingModule
import PyUtilModule.EnvUtils as EnvUtils
import PyFnGeolibServices.ExpressionMath as ExpressionMath
import PyUtilModule.FarmAPI as FarmAPI
import PyUtilModule.FileUtils as FileUtils
import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import PyFnGeolibServices as FnGeolibServices
import GeoAPI as GeoAPI
import PyUtilModule.Hints as Hints
import PyUtilModule.IRFs as IRFs
import PyUtilModule.KatanaFile as KatanaFile
import KatanaResources as KatanaResources
import PyUtilModule.LiveRenderAPI as LiveRenderAPI
import MachineInfo as MachineInfo
import MediaCacheHandler as MediaCacheHandler
import Naming as Naming
import PyUtilModule.NodeDebugOutput as NodeDebugOutput
import NodeGraphView as NodeGraphView
import NodegraphAPI as NodegraphAPI
import Nodes2DAPI as Nodes2DAPI
import Nodes3DAPI as Nodes3DAPI
import PyUtilModule.NonUIPluginManager as NonUIPluginManager
import PyOpenColorIO as OCIO
import PyUtilModule.OpDocumentationGenerator as OpDocumentationGenerator
import PluginSystemAPI as PluginSystemAPI
import PyUtilModule.ProjectSnapshot as ProjectSnapshot
import PyFCurve as PyFCurve
import PyXmlIO as PyXmlIO
import QT4Browser as QT4Browser
import QT4Color as QT4Color
import QT4FormWidgets as QT4FormWidgets
import QT4GLLayerStack as QT4GLLayerStack
import QT4Panels as QT4Panels
import QT4Widgets as QT4Widgets
import QTFCurve as QTFCurve
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import PyUtilModule.RegisterToCamera as RegisterToCamera
import PyUtilModule.RenderManager as RenderManager
import RenderingAPI as RenderingAPI
import PyUtilModule.RenderingCommon as RenderingCommon
import RerenderEventMapper as RerenderEventMapper
import PyResolutionTableFn as ResolutionTable
import KatanaResources as ResourceFiles
import PyUtilModule.ScenegraphBookmarkManager as ScenegraphBookmarkManager
import Nodes3DAPI.ScenegraphManager as ScenegraphManager
import PyUtilModule.ScenegraphUtils as ScenegraphUtils
import PyUtilModule.Shelves as Shelves
import PyUtilModule.StartupScripts as StartupScripts
import PyUtilModule.SuperToolPlugins as SuperToolPlugins
import PyUtilModule.UndoEntries as UndoEntries
import Naming as UniqueName
import PyUtilModule.UserNodes as UserNodes
import Utils as Utils
import WorkQueue as WorkQueue
import PyUtilModule.WorkingSet as WorkingSet
import PyUtilModule.WorkingSetClient as WorkingSetClient
import PyUtilModule.WorkingSetManager as WorkingSetManager
from PyUtilModule import Initialize as Initialize
from typing import Set, Tuple

LogGLHandlersOldLevel: int



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import AssetAPI as AssetAPI
import CacheManager as CacheManager
import NodegraphAPI.CallbackTypes as CallbackTypes
import Callbacks as Callbacks
import ConfigurationAPI_cmodule as Configuration
import PyFnGeolibServices.ExpressionMath as ExpressionMath
import PyFnAttribute as FnAttribute
import PyFnGeolibServices as FnGeolibServices
import PyOpenColorIO as OCIO
import PyFCurve as PyFCurve
import PyXmlIO as PyXmlIO
import PyResolutionTableFn as ResolutionTable
import Naming as UniqueName
import Utils as Utils
import PyXmlIO as XmlIO
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import KatanaResources as KatanaResources
import NodegraphAPI as NodegraphAPI
import Utils as Utils
from typing import Set, Tuple

def ParseMacroFilename(filename: str) -> Tuple[str, str]: ...
def PublishNode(node: NodegraphAPI.Node, filename: str): ...
def ReloadCustomNodes(): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from typing import Set, Tuple

class CustomMenu(PyQt5.QtWidgets.QMenu):
    def __init__(self, parent) -> None: ...
    def actionEvent(self, event: PyQt5.QtGui.QActionEvent): ...
    def sizeHint(self) -> PyQt5.QtCore.QSize: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Util.IconManager as IconManager
import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class RecordItemModel(PyQt5.QtGui.QStandardItemModel):
    recordAdded: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    recordsRemoved: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, columnTitles: list[str], lowestRecordLevel: int, parent: Incomplete | None = ...) -> None: ...
    def _RecordItemModel__on_aboutToQuit(self): ...
    def addRecord(self, record: Record) -> bool: ...
    def data(self, modelIndex: PyQt5.QtCore.QModelIndex, role: PyQt5.QtCore.Qt.ItemDataRole = ...) -> PyQt5.QtCore.QVariant: ...
    def flags(self, modelIndex: PyQt5.QtCore.QModelIndex) -> PyQt5.QtCore.Qt.ItemFlags: ...
    @staticmethod
    def formatTimestamp(timestamp: float) -> str: ...
    def getHighestRecordLevel(self) -> int: ...
    def getRecordLevels(self) -> set[int]: ...
    def removeRecord(self, record: Record) -> bool: ...
    def removeRows(self, row: int, count: int, parentIndex: PyQt5.QtCore.QModelIndex) -> bool: ...
    @staticmethod
    def timestampToDateTime(timestamp: float) -> PyQt5.QtCore.QDateTime: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.FormMaster.EnableableParameterPolicy as EnableableParameterPolicy
import UI4.FormMaster.HintsDelegate as HintsDelegate
import UI4.FormMaster.ParameterPolicy as ParameterPolicy
from UI4.FormMaster.NodeHints.Common2D import AddCommon2DToNodeHints as AddCommon2DToNodeHints
from typing import Set, Tuple

AnisotropicParamHints: dict
BorderExtendParamHints: dict
BorderOverscanParamHints: dict
ClampOutputParamHints: dict
FilterParamHints: dict
GroupFilterParamHints: dict
HighlightCompensationParamHints: dict
_NodeHints: dict
_NodeName: str



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import NodegraphAPI as NodegraphAPI
from Nodes3DAPI.Node3D import Node3D as Node3D
from typing import ClassVar, Set, Tuple

_ExtraHints: dict
_Parameter_XML: str

class BoundsAdjust(Node3D):
    ACTION_NAME_MAP: ClassVar[dict] = ...
    ACTION_PAD_LOCAL: ClassVar[str] = ...
    ACTION_PAD_PERC: ClassVar[str] = ...
    ACTION_STRIP: ClassVar[str] = ...
    VALID_ACTIONS: ClassVar[list] = ...
    def __init__(self) -> None: ...
    def _getOpChain(self, interface): ...
    def addParameterHints(self, attrName, inputDict): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Utils.EventModuleCommon as EventModuleCommon
import typing
from Utils.Exceptions import GetExceptionMessage as GetExceptionMessage
from typing import Set, Tuple

_EventQueue: list
_SynchronousEventProcessing: bool

def Initialize(): ...
def ProcessAllEvents(): ...
def ProcessEvents() -> bool: ...
def PumpIdleQueue(): ...
def QueueEvent(eventType: str, eventID: typing.Hashable, priority: int = ..., **kwargs): ...
def QueuePriorityEvent(eventType: str, eventID: typing.Hashable, **kwargs): ...
def _handleEvents(events: list[tuple]): ...
def _invokeCollapsedDict(mask: tuple[str, object], args: list[tuple[str, object, dict]]): ...
def _invokeHandlerDict(mask: tuple[str, object], eventType: str, eventID: object, kwargs: dict): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import CatalogAPI as CatalogAPI
import CatalogWidget as CatalogWidget
import NodegraphAPI as NodegraphAPI
import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import Utils as Utils
import collections
import enum
from CatalogAPI.CatalogItem import QueueCatalogItemPropertyUpdate as QueueCatalogItemPropertyUpdate
from QT4Widgets.SortableTreeWidget import SortableTreeWidget as SortableTreeWidget
from UI4.Tabs.Catalog.CatalogWidget.CatalogWidget import kColumnTypes
from UI4.Tabs.Catalog.CatalogWidgetItem import CatalogSlotParentItem as CatalogSlotParentItem, CatalogWidgetItem as CatalogWidgetItem, CatalogWidgetItemDelegate as CatalogWidgetItemDelegate, GetColumnID as GetColumnID, GetColumnKeyName as GetColumnKeyName, GetColumnName as GetColumnName, GetDefaultColumnWidth as GetDefaultColumnWidth
from UI4.Util.CatalogEventHandlers import RegenerateThumbnail as RegenerateThumbnail
from UI4.Util.Events import debounce as debounce
from Utils.Decorators import deprecated as deprecated
from typing import ClassVar, Set, Tuple

class CatalogWidget(SortableTreeWidget):
    class kColumnTypes(enum.Enum):
        Default: ClassVar[kColumnTypes] = ...
        GSV: ClassVar[kColumnTypes] = ...
        IRF: ClassVar[kColumnTypes] = ...
        _member_map_: ClassVar[collections.OrderedDict] = ...
        _member_names_: ClassVar[list] = ...
        _member_type_: ClassVar[type[object]] = ...
        _value2member_map_: ClassVar[dict] = ...
        @classmethod
        def __init__(cls, value) -> None: ...
        def _generate_next_value_(self, name, start, count, last_values): ...
    LIST_ALL: ClassVar[int] = ...
    LIST_LOCKED: ClassVar[int] = ...
    LIST_SELECTED: ClassVar[int] = ...
    LIST_UNLOCKED: ClassVar[int] = ...
    _CatalogWidget__headerState: ClassVar[None] = ...
    _kDefaultColumnWidthForDynamicColumns: ClassVar[int] = ...
    _kDynamicColumnTooltip: ClassVar[dict] = ...
    _kHeaderTooltips: ClassVar[dict] = ...
    _kHideableHeaderTooltip: ClassVar[str] = ...
    _kMoveableHeaderTooltip: ClassVar[str] = ...
    kColumnTypeRole: ClassVar[PyQt5.QtCore.Qt.ItemDataRole] = ...
    def __init__(self, *args) -> None: ...
    def _CatalogWidget__aboutToDrag(self, dragItems, dragObject): ...
    def _CatalogWidget__areTopLevelItemsReadyForSlotMode(self): ...
    def _CatalogWidget__areTopLevelItemsReadyForTimeMode(self): ...
    def _CatalogWidget__catalog_clientSlotUpdate_CB(self, *args, **kwargs): ...
    def _CatalogWidget__catalog_itemCreate_CB(self, eventType, index, item, **kwargs): ...
    def _CatalogWidget__catalog_itemDelete_CB(self, args): ...
    def _CatalogWidget__catalog_itemPropertyUpdate_CB(self, eventType, index, item, **kwargs): ...
    def _CatalogWidget__catalog_itemSlotUpdate_CB(self, eventType, index, item, **kwargs): ...
    def _CatalogWidget__catalog_itemUserCommentUpdate_CB(self, eventType, index, item, **kwargs): ...
    def _CatalogWidget__catalog_itemsReordered_CB(self, args): ...
    def _CatalogWidget__catalog_rebuild_CB(self, args): ...
    def _CatalogWidget__columnMovedCB(self, section, fromIndex, toIndex): ...
    def _CatalogWidget__dragMoveEvent(self, event, parent, index, callbackRecord): ...
    def _CatalogWidget__dropEvent(self, event, parent, index): ...
    def _CatalogWidget__findListViewItemFromCatalogItem(self, catalogItem): ...
    def _CatalogWidget__getSlotItem(self, index): ...
    def _CatalogWidget__headerSignalBlocker(self, *args, **kwds): ...
    def _CatalogWidget__keypressCallback(self, event): ...
    def _CatalogWidget__mousePressEvent_CB(self, event): ...
    def _CatalogWidget__onProjectSettingsChange(self, eventData: list[tuple[str, object, dict]]): ...
    def _CatalogWidget__regenerateThumbnails(self, *args, **kwargs): ...
    def _CatalogWidget__saveProjectSettingsDebounced(self, *args, **kwargs): ...
    def _CatalogWidget__sectionResized_CB(self, column, oldSize, newSize): ...
    def _CatalogWidget__setHeaderTooltips(self): ...
    def _CatalogWidget__updateThumbnailPixmaps(self, width: int): ...
    def _CatalogWidget__updateThumbnailWidth(self): ...
    def addDynamicColumn(self, columnName: str, columnType: CatalogWidget.kColumnTypes): ...
    def applyColumnHiddenSettings(self, columnsHidden: dict[str, bool]): ...
    def applyColumnOrderSettings(self, columnOrder: dict[str, int]): ...
    def applyColumnWidthsSettings(self, columnWidths: dict[str, int]): ...
    def applyProjectSettings(self): ...
    def deleteItems(self, selectionMask): ...
    def drawBranches(self, painter, rect, index): ...
    def getColumnHiddenSettings(self) -> dict[str, bool]: ...
    def getColumnOrderSettings(self) -> dict[str, int]: ...
    def getColumnWidthsSettings(self) -> dict[str, int]: ...
    def getItems(self, selectionMask): ...
    def getMonitorSwipeMode(self): ...
    def getSmallFont(self): ...
    def rebuild(self): ...
    def restoreState(self): ...
    def saveProjectSettings(self): ...
    def saveState(self): ...

class _CatalogThumbnailWidthRequester(PyQt5.QtCore.QObject):
    _CatalogThumbnailWidthRequester__instance: ClassVar[None] = ...
    thumbnailSizeReset: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self) -> None: ...
    @classmethod
    def instance(cls): ...
    def requestWidth(self, width: int): ...
    def resetWidth(self): ...

class _HeaderContextMenu(_NonAutomaticallyDissmisableMenu):
    def __init__(self, parent: CatalogWidget) -> None: ...
    def _HeaderContextMenu__onChecked(self): ...
    def open(self, point: PyQt5.QtCore.QPoint): ...

class _HeaderGSVMenu(_HeaderSubMenuBase):
    _kColumnType: ClassVar[kColumnTypes] = ...
    _kCombinedColumnIdx: ClassVar[int] = ...
    def __init__(self, parent) -> None: ...
    def _getColumnNamesFromSnapshot(self, snapshot: ProjectSnapshot) -> list[str]: ...

class _HeaderIRFMenu(_HeaderSubMenuBase):
    _kColumnType: ClassVar[kColumnTypes] = ...
    _kCombinedColumnIdx: ClassVar[int] = ...
    def __init__(self, parent) -> None: ...
    def _getColumnNamesFromSnapshot(self, snapshot: ProjectSnapshot) -> list[str]: ...

class _HeaderSubMenuBase(_NonAutomaticallyDissmisableMenu):
    _kColumnType: ClassVar[NotImplementedType] = ...
    _kCombinedColumnIdx: ClassVar[NotImplementedType] = ...
    def __init__(self, parent: _HeaderContextMenu) -> None: ...
    def _HeaderSubMenuBase__createAddColumnAction(self, label: str) -> PyQt5.QtWidgets.QAction: ...
    def _HeaderSubMenuBase__createCheckboxAction(self, label: str) -> PyQt5.QtWidgets.QAction: ...
    def _HeaderSubMenuBase__createExistingColumnAction(self, label: str, logicalIdx: int) -> PyQt5.QtWidgets.QAction: ...
    @classmethod
    def _HeaderSubMenuBase__createNew(cls, parent): ...
    def _HeaderSubMenuBase__onAddNew(self): ...
    def _HeaderSubMenuBase__onToggleExisting(self): ...
    def _getColumnNamesFromSnapshot(self, snapshot: ProjectSnapshot): ...

class _NonAutomaticallyDissmisableMenu(PyQt5.QtWidgets.QMenu):
    def event(self, event: PyQt5.QtCore.QEvent) -> bool: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.FormMaster.Editors.HideTitleGroup as HideTitleGroup
import UI4.FormMaster.Editors.Number as Number
import QT4Color as QT4Color
import QT4FormWidgets as QT4FormWidgets
import QT4FormWidgets.ValuePolicy
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import UI4.FormMaster.Editors.HideTitleGroup
import UI4.FormMaster.Editors.Number
import Utils as Utils
from QT4FormWidgets.GroupFormWidget import GroupFormWidget
from typing import Set, Tuple

class ColorSliderGroupFormWidget(GroupFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _ColorSliderGroupFormWidget__reset_CB(self): ...
    def _ColorSliderGroupFormWidget__setColorRawRGB(self, rgbColor, final: bool = ...): ...
    def _ColorSliderGroupFormWidget__updateColorWidget(self): ...
    def _ColorSliderGroupFormWidget__updateLockState(self): ...
    def _getColorPolicies(self): ...
    def _lockChanged(self, state): ...
    def _popdownCreated(self, popdown): ...
    def getPopdownWidget(self): ...
    def showPopdown(self, value): ...
    def valueChangedEvent(self, event): ...

class ColorWheelAndLumaSliderGroupFormWidget(UI4.FormMaster.Editors.HideTitleGroup.HideTitleGroupFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _ColorWheelAndLumaSliderGroupFormWidget__lumaChanged_CB(self, luma, final: bool = ...): ...
    def _ColorWheelAndLumaSliderGroupFormWidget__lumaValueChanged(self, event): ...
    def _ColorWheelAndLumaSliderGroupFormWidget__reset_CB(self): ...
    def _ColorWheelAndLumaSliderGroupFormWidget__setColorRawRGB(self, rgbColor, final: bool = ...): ...
    def _ColorWheelAndLumaSliderGroupFormWidget__updateColorWidget(self): ...
    def _ColorWheelAndLumaSliderGroupFormWidget__updateLockState(self): ...
    def _ColorWheelAndLumaSliderGroupFormWidget__updateLumaWidget(self): ...
    def _ColorWheelAndLumaSliderGroupFormWidget__valueChanged(self, event): ...
    def _createChildWidget(self, *args): ...
    def _freeze(self): ...
    def _lockChanged(self, state): ...
    def _popdownCreated(self, popdown): ...
    def _thaw(self): ...
    def showPopdown(self, value): ...

class ColorWheelGroupFormWidget(UI4.FormMaster.Editors.HideTitleGroup.HideTitleGroupFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _ColorWheelGroupFormWidget__reset_CB(self): ...
    def _ColorWheelGroupFormWidget__setColorRawRGB(self, rgbColor, final: bool = ...): ...
    def _ColorWheelGroupFormWidget__updateColorWidget(self): ...
    def _ColorWheelGroupFormWidget__updateLockState(self): ...
    def _ColorWheelGroupFormWidget__valueChanged(self, event): ...
    def _freeze(self): ...
    def _getColorPolicies(self): ...
    def _lockChanged(self, state): ...
    def _popdownCreated(self, popdown): ...
    def _thaw(self): ...
    def getPopdownWidget(self): ...
    def showPopdown(self, value): ...

class UnitsNumberWidget(UI4.FormMaster.Editors.Number.KatanaNumberFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _UnitsNumberWidget__updateInfo(self): ...
    def _UnitsNumberWidget__valueChanged(self, event): ...
    def _freeze(self): ...
    def _thaw(self): ...

class _HideStateValuePolicyProxy(QT4FormWidgets.ValuePolicy.ValuePolicyProxy):
    def shouldDisplayState(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Utils as Utils
from typing import Set, Tuple

class UndoContextGuard:
    def __init__(self, name, logWarnings: bool = ...) -> None: ...
    def __enter__(self): ...
    def __exit__(self, excType, excValue, tb): ...

class UndoGrouping:
    def __init__(self, name) -> None: ...
    def __del__(self) -> None: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import KatanaResources as KatanaResources
import PyQt5.QtWidgets
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import Utils as Utils
from UI4.Tabs.BaseTab import BaseTab as BaseTab
from UI4.Tabs.UVPanel.UVWidget import UVWidget as UVWidget
from UI4.Widgets.StatefulMenu import MergeStatefulMenuData as MergeStatefulMenuData, StatefulMenu as StatefulMenu
from typing import Set, Tuple

class UVPanel(BaseTab):
    def __init__(self, *args) -> None: ...
    def _UVPanel__buildMenu(self): ...
    def _UVPanel__on_textureCombobox_activated(self, index): ...
    def _UVPanel__on_tileCombobox_activated(self, index): ...
    def _UVPanel__openTextureFile(self): ...
    def _UVPanel__updateTexturesCallback(self, textureList): ...
    def _UVPanel__updateTilesCallback(self, bounds): ...
    def getIsolateSelection(self): ...
    def getMenuBar(self) -> PyQt5.QtWidgets.QMenuBar | None: ...
    def getOverlaySetting(self, name): ...
    def getStencilMode(self): ...
    def getTileFormat(self): ...
    def getUVWidget(self) -> UVWidget: ...
    def setIsolateSelection(self, value): ...
    def setOverlaySetting(self, name, value): ...
    def setStencilMode(self, value): ...
    def setTileFormat(self, tileFormat): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Nodes3DAPI.AbstractTransform as AT
import ConfigurationAPI_cmodule as Configuration
import PyFnAttribute as FnAttribute
import Nodes3DAPI.Node3D as Node3D
import NodegraphAPI as NodegraphAPI
import PyFnGeolibProducers
import Nodes3DAPI.TransformUtil as TransformUtil
from Nodes3DAPI.AbstractTransform import AbstractTransform
from typing import Set, Tuple

_ExtraHints: dict
_Parameter_XML: str
_resource_path: str

class BlockerCreate(AbstractTransform):
    def __init__(self) -> None: ...
    def _BlockerCreate__addOverride(self, producer: PyFnGeolibProducers.GeometryProducer, coordSysName, attrType, name, value): ...
    def addParameterHints(self, attrName, inputDict): ...
    def getScenegraphLocation(self): ...
    def setInteractiveTransform(self, path, absScale, absRotate, absTranslate, time): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Widgets.MessageBox as MessageBox
import Utils as Utils
from typing import Set, Tuple

def ExecScript(valuePolicy, buttonText, scriptText): ...
def GetButtonAndScriptText(valuePolicy): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from . import DarkMojo as DarkMojo, Manifest as Manifest, Profile as Profile, PythonConsole as PythonConsole, QtAttributes as QtAttributes, Settings as Settings, TextEditorUtils as TextEditorUtils, Threads as Threads, Timebar as Timebar, Timeline as Timeline, TopWindows as TopWindows, TreeWidgetUtil as TreeWidgetUtil, WidgetUtils as WidgetUtils
from QT4Widgets.CapsuleCombo import CapsuleCombo as CapsuleCombo, CapsuleComboBase as CapsuleComboBase, TransitionValidator as TransitionValidator
from QT4Widgets.CustomMenu import CustomMenu as CustomMenu
from QT4Widgets.CustomQLineEdit import CustomQLineEdit as CustomQLineEdit
from QT4Widgets.DarkMojo import DarkMojoPalette as DarkMojoPalette
from QT4Widgets.ExpandingLabel import ExpandingLabel as ExpandingLabel
from QT4Widgets.FilterFieldWidget import FilterFieldWidget as FilterFieldWidget
from QT4Widgets.FilterablePopupButton import FilterableCombo as FilterableCombo, FilterablePopup as FilterablePopup, FilterablePopupButton as FilterablePopupButton
from QT4Widgets.InteractiveIconTabBar import InteractiveIconTabBar as InteractiveIconTabBar
from QT4Widgets.LogView import LogView as LogView
from QT4Widgets.MenuButton import MenuButton as MenuButton
from QT4Widgets.NavigationToolbar import NavigationToolbar as NavigationToolbar
from QT4Widgets.PopdownLabel import PopdownLabel as PopdownLabel
from QT4Widgets.PopupMenuOption import PopupMenuOption as PopupMenuOption
from QT4Widgets.Profile import ProfileStatsWidget as ProfileStatsWidget, main as main
from QT4Widgets.PythonConsole import FullInteractivePython as FullInteractivePython, InteractivePython as InteractivePython
from QT4Widgets.SliderWidget import SliderBase as SliderBase, SliderWidget as SliderWidget
from QT4Widgets.SortableTreeWidget import CallbackRecord as CallbackRecord, SortableTreeWidget as SortableTreeWidget, SortableTreeWidgetItem as SortableTreeWidgetItem, SortableTreeWidgetItemDelegate as SortableTreeWidgetItemDelegate
from QT4Widgets.StretchBox import StretchBox as StretchBox
from QT4Widgets.ToolbarButton import LabeledToolbarButton as LabeledToolbarButton, ToolbarButton as ToolbarButton
from QT4Widgets.TopWindows import TrackTopWindow as TrackTopWindow
from QT4Widgets.TreeWidgetUtil import ContextMenuHelper as ContextMenuHelper, UpdateSuppressor as UpdateSuppressor
from QT4Widgets.VerticalDivider import VerticalDivider as VerticalDivider
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore as QtCore
from PyQt5.QtCore import CursorChange as CursorChange, WA_DontCreateNativeAncestors as WA_DontCreateNativeAncestors, WA_NativeWindow as WA_NativeWindow, WidgetWithChildrenShortcut as WidgetWithChildrenShortcut
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import sip
from typing import Set, Tuple

class Foundry(sip.simplewrapper):
    class UI(sip.simplewrapper):
        class FileBrowser(PyQt5.QtWidgets.QWidget):
            @classmethod
            def AddFavoriteDir(cls, *args, **kwargs): ...
            @classmethod
            def RemoveFavoriteDir(cls, *args, **kwargs): ...
            @classmethod
            def ResetFavorites(cls, *args, **kwargs): ...
            @classmethod
            def ResetNonPersistentFavorites(cls, *args, **kwargs): ...
            @classmethod
            def SetDefaultFavoritePaths(cls, *args, **kwargs): ...
            @classmethod
            def SetPreferencesPath(cls, *args, **kwargs): ...
            @classmethod
            def actionEvent(cls, *args, **kwargs): ...
            @classmethod
            def changeEvent(cls, *args, **kwargs): ...
            @classmethod
            def childEvent(cls, *args, **kwargs): ...
            @classmethod
            def closeEvent(cls, *args, **kwargs): ...
            @classmethod
            def connectNotify(cls, *args, **kwargs): ...
            @classmethod
            def contextMenuEvent(cls, *args, **kwargs): ...
            @classmethod
            def create(cls, *args, **kwargs): ...
            @classmethod
            def customEvent(cls, *args, **kwargs): ...
            @classmethod
            def destroy(cls, *args, **kwargs): ...
            @classmethod
            def disconnectNotify(cls, *args, **kwargs): ...
            @classmethod
            def dragEnterEvent(cls, *args, **kwargs): ...
            @classmethod
            def dragLeaveEvent(cls, *args, **kwargs): ...
            @classmethod
            def dragMoveEvent(cls, *args, **kwargs): ...
            @classmethod
            def dropEvent(cls, *args, **kwargs): ...
            @classmethod
            def enterEvent(cls, *args, **kwargs): ...
            @classmethod
            def event(cls, *args, **kwargs): ...
            @classmethod
            def focusInEvent(cls, *args, **kwargs): ...
            @classmethod
            def focusNextChild(cls, *args, **kwargs): ...
            @classmethod
            def focusNextPrevChild(cls, *args, **kwargs): ...
            @classmethod
            def focusOutEvent(cls, *args, **kwargs): ...
            @classmethod
            def focusPreviousChild(cls, *args, **kwargs): ...
            @classmethod
            def hideEvent(cls, *args, **kwargs): ...
            @classmethod
            def initPainter(cls, *args, **kwargs): ...
            @classmethod
            def inputMethodEvent(cls, *args, **kwargs): ...
            @classmethod
            def isSignalConnected(cls, *args, **kwargs): ...
            @classmethod
            def keyPressEvent(cls, *args, **kwargs): ...
            @classmethod
            def keyReleaseEvent(cls, *args, **kwargs): ...
            @classmethod
            def leaveEvent(cls, *args, **kwargs): ...
            @classmethod
            def metric(cls, *args, **kwargs): ...
            @classmethod
            def mouseDoubleClickEvent(cls, *args, **kwargs): ...
            @classmethod
            def mouseMoveEvent(cls, *args, **kwargs): ...
            @classmethod
            def mousePressEvent(cls, *args, **kwargs): ...
            @classmethod
            def mouseReleaseEvent(cls, *args, **kwargs): ...
            @classmethod
            def moveEvent(cls, *args, **kwargs): ...
            @classmethod
            def nativeEvent(cls, *args, **kwargs): ...
            @classmethod
            def paintEvent(cls, *args, **kwargs): ...
            @classmethod
            def receivers(cls, *args, **kwargs): ...
            @classmethod
            def refreshFavorites(cls, *args, **kwargs): ...
            @classmethod
            def resizeEvent(cls, *args, **kwargs): ...
            @classmethod
            def selectedFiles(cls, *args, **kwargs): ...
            @classmethod
            def sender(cls, *args, **kwargs): ...
            @classmethod
            def senderSignalIndex(cls, *args, **kwargs): ...
            @classmethod
            def setButtonsVisible(cls, *args, **kwargs): ...
            @classmethod
            def setDirectory(cls, *args, **kwargs): ...
            @classmethod
            def setFileSequenceEvaluator(cls, *args, **kwargs): ...
            @classmethod
            def setFilename(cls, *args, **kwargs): ...
            @classmethod
            def setFilters(cls, *args, **kwargs): ...
            @classmethod
            def setMayNotExist(cls, *args, **kwargs): ...
            @classmethod
            def setSequencesEnabled(cls, *args, **kwargs): ...
            @classmethod
            def setShowChooserTypes(cls, *args, **kwargs): ...
            @classmethod
            def sharedPainter(cls, *args, **kwargs): ...
            @classmethod
            def showEvent(cls, *args, **kwargs): ...
            @classmethod
            def tabletEvent(cls, *args, **kwargs): ...
            @classmethod
            def timerEvent(cls, *args, **kwargs): ...
            @classmethod
            def updateMicroFocus(cls, *args, **kwargs): ...
            @classmethod
            def wheelEvent(cls, *args, **kwargs): ...

        class FileInfo(sip.wrapper):
            @classmethod
            def fileName(cls, *args, **kwargs): ...
            @classmethod
            def insertSequenceFile(cls, *args, **kwargs): ...
            @classmethod
            def setEndFrame(cls, *args, **kwargs): ...
            @classmethod
            def setSequenceName(cls, *args, **kwargs): ...
            @classmethod
            def setStartFrame(cls, *args, **kwargs): ...

        class FileSequenceEvaluator(sip.wrapper):
            @classmethod
            def buildFileSequence(cls, *args, **kwargs): ...
            @classmethod
            def evaluateFiles(cls, *args, **kwargs): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from QT4Widgets.PopdownLabel import PopdownLabel as PopdownLabel
from typing import ClassVar, Set, Tuple

class LabelButton(PyQt5.QtWidgets.QLabel):
    mousePressEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def mousePressEvent(self, ev): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from typing import ClassVar, Set, Tuple

class LumaSliderWidget(PyQt5.QtWidgets.QWidget):
    valueChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent) -> None: ...
    def _LumaSliderWidget__getCentralRect(self): ...
    def colorToWindowPos(self, value): ...
    def enterEvent(self, ev): ...
    def getLuma(self): ...
    def isPreselectionActive(self): ...
    def leaveEvent(self, ev): ...
    def minimumSizeHint(self): ...
    def mouseMoveEvent(self, ev): ...
    def mousePressEvent(self, ev): ...
    def mouseReleaseEvent(self, ev): ...
    def paintEvent(self, ev): ...
    def reset(self): ...
    def setLuma(self, luma): ...
    def sizeHint(self): ...
    def sizePolicy(self): ...
    def windowPosToColor(self, pos): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import QT4FormWidgets as QT4FormWidgets
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
from UI4.Tabs.BaseTab import BaseTab as BaseTab
from typing import Set, Tuple

_HelpText: str

class CELPanel(PyQt5.QtWidgets.QScrollArea):
    def __init__(self, parent) -> None: ...

class CELTab(BaseTab):
    def __init__(self, parent) -> None: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyOpenColorIO as OCIO
import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import QT4Color as QT4Color
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import UI4.FormMaster.Editors.BaseRamp
import Utils as Utils
import collections
from UI4.FormMaster.Editors.BaseRamp import BaseRampFormWidget as BaseRampFormWidget
from UI4.FormMaster.Editors.SortableAttributeDropEditor import RegisterCoPolicyWidgetHintKey as RegisterCoPolicyWidgetHintKey
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class ColorRampFormWidget(UI4.FormMaster.Editors.BaseRamp.BaseRampFormWidget):
    Presets: ClassVar[collections.OrderedDict] = ...
    ValuesTypeString: ClassVar[str] = ...
    def __init__(self, parent, policy, factory) -> None: ...
    def _ColorRampFormWidget__on_gradientWidget_rampChanged(self): ...
    def _ColorRampFormWidget__on_menu_aboutToShow(self): ...
    def _ColorRampFormWidget__on_presetAction_triggered(self): ...
    def _ColorRampFormWidget__sensitivityMenuCallback(self, action): ...
    def _ColorRampFormWidget__updateFilmlookActionText(self): ...
    def _addControlWidgets(self, knots: list[tuple], interpolator: str, interpolatorOptions: list[str], layout: PyQt5.QtWidgets.QHBoxLayout): ...
    def _lockChanged(self, state: bool): ...
    def _updateControlWidgets(self, knots: list[tuple], interpolator: str): ...

class GradientRender(PyQt5.QtWidgets.QWidget):
    def __init__(self, filmlook, width: int = ..., height: int = ..., parent: Incomplete | None = ...) -> None: ...
    def getColorAt(self, pos: PyQt5.QtGui.QPoint) -> PyQt5.QtGui.QColor: ...
    def paintEvent(self, e): ...
    def setFilmlookEnabled(self, value): ...
    def setKnots(self, knots: list[Knot], interp: str = ...): ...

class GradientWidget(PyQt5.QtWidgets.QWidget):
    rampChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, knots, interp, height: int = ..., useFilmlook: bool = ..., restrictComponents: bool = ..., parent: Incomplete | None = ...) -> None: ...
    def _GradientWidget__drawPositionLabel(self, knot): ...
    def _GradientWidget__hidePositionLabel(self): ...
    def _GradientWidget__knot_point(self, gpos): ...
    def _GradientWidget__on_xLabelVisibilityTimer_timeout(self): ...
    def addKnot(self, pos, color, render: bool = ...): ...
    def getInterp(self): ...
    def getKnotSensitivity(self): ...
    def hideEvent(self, event: PyQt5.QtGui.QHideEvent): ...
    def knotChanged(self, final: bool = ...): ...
    def knotClicked(self, knot): ...
    def knotDragged(self, knot, xmove, x): ...
    def knots(self): ...
    def mouseMoveEvent(self, event: PyQt5.QtGui.QMouseEvent): ...
    def mousePressEvent(self, event: PyQt5.QtGui.QMouseEvent): ...
    def mouseReleaseEvent(self, event: PyQt5.QtGui.QMouseEvent): ...
    def removeKnot(self, knot): ...
    def render(self): ...
    def replaceKnots(self, knots, emit: bool = ...): ...
    def resizeEvent(self, event): ...
    def setFilmlookEnabled(self, value): ...
    def setInterp(self, interp): ...
    def setKnotSensitivity(self, sensitivity): ...

class Knot(PyQt5.QtWidgets.QWidget):
    clicked: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    knotChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    knotRemoved: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    positionChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, position, color, useFilmLook: bool = ..., restrictComponents: bool = ..., parent: Incomplete | None = ...) -> None: ...
    def _Knot__remove(self): ...
    def _Knot__updateBrush(self): ...
    def color(self): ...
    def colorChange(self, color): ...
    def contextMenuEvent(self, event): ...
    def isDown(self): ...
    def keyPressEvent(self, event): ...
    def mouseDoubleClickEvent(self, event): ...
    def mouseMoveEvent(self, event): ...
    def mousePressEvent(self, event): ...
    def mouseReleaseEvent(self, event): ...
    def paintEvent(self, ev): ...
    def position(self): ...
    def setColor(self, color): ...
    def setDown(self, value): ...
    def setFilmlookEnabled(self, value): ...
    def setPosition(self, position): ...
    def sizeHint(self): ...

class KnotToolTip(PyQt5.QtWidgets.QLabel):
    def __init__(self, parent: Incomplete | None = ...) -> None: ...

def EvalSpline(u, knots): ...
def clamp(minvalue, maxvalue, value): ...
def createQColor(c, filmLook: bool = ...): ...
def fit(val, inmin, inmax, outmin, outmax): ...
def lerp(mix, a, b): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI
import PyXmlIO as PyXmlIO
import NodegraphAPI.Xio as Xio
from typing import Set, Tuple

def CreateGroupParameter(node: NodegraphAPI.Node, delimitedParameterName): ...
def CreateNumberParameter(node: NodegraphAPI.Node, delimitedParameterName): ...
def CreateStringParameter(node: NodegraphAPI.Node, delimitedParameterName): ...
def DeleteParameter(node: NodegraphAPI.Node, delimitedParameterName): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import AssetAPI as AssetAPI
import UI4.Widgets.MessageBox as MessageBox
import NodegraphAPI as NodegraphAPI
import PyFCurve as PyFCurve
import QT4Browser as QT4Browser
import QT4FormWidgets as QT4FormWidgets
import QT4FormWidgets.ValuePolicy
import QTFCurve as QTFCurve
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from QT4FormWidgets.FormWidget import FormWidget
from QT4Widgets.MenuButton import MenuButton as MenuButton
from QT4Widgets.StretchBox import StretchBox as StretchBox
from QTFCurve.FCurveGraphValueListWidget import FCurveGraphValueListWidget
from UI4.FormMaster.Editors.AssetId import AssetIdWidget as AssetIdWidget
from UI4.Util.AssetId import BrowseForAsset as BrowseForAsset
from typing import Set, Tuple

class FCurveFormWidget(FormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _FCurveFormWidget__actionMenuAboutToShow(self): ...
    def _FCurveFormWidget__configureEditor(self): ...
    def _FCurveFormWidget__curveChanged(self, curve, force: bool = ...): ...
    def _FCurveFormWidget__exportFcurve(self): ...
    def _FCurveFormWidget__fileEditorActive(self): ...
    def _FCurveFormWidget__importFcurve(self): ...
    def _FCurveFormWidget__setControlAreaLabel(self): ...
    def _FCurveFormWidget__subscribeContainer(self): ...
    def _FCurveFormWidget__switchToFCurve(self): ...
    def _FCurveFormWidget__switchToFileName(self): ...
    def _FCurveFormWidget__switchToMulti(self): ...
    def _FCurveFormWidget__unsubscribeContainer(self): ...
    def _buildControlWidget(self, hbox): ...
    def _thaw(self): ...
    def valueChangedEvent(self, event): ...

class _CurveEditor(FCurveGraphValueListWidget):
    def __init__(self, ratio, *args) -> None: ...
    def heightForWidth(self, w): ...

class _FileNamePolicyProxy(QT4FormWidgets.ValuePolicy.ValuePolicyProxy):
    def __init__(self, policy, hints) -> None: ...
    def getWidgetHints(self): ...
    def shouldDisplayState(self): ...
    def shouldDisplayWrench(self): ...

class _SimpleContainerObserver(PyFCurve.ContainerObserver):
    def __init__(self, observer, policy, setLabelFunc) -> None: ...
    def containerChanged(self, container): ...

class _SimpleObserver(PyFCurve.FCurveObserver):
    def __init__(self) -> None: ...
    def beginValueChange(self, curve): ...
    def curveChanged(self, curve): ...
    def emitter(self): ...
    def endValueChange(self, curve): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import AbstractSyntaxHighlighter as AbstractSyntaxHighlighter, AnamorphDisplayOptions as AnamorphDisplayOptions, AssetId as AssetId, AssetWidgetDelegatePlugins as AssetWidgetDelegatePlugins, AttributeHistory as AttributeHistory, BuiltinNotifications as BuiltinNotifications, CMakeSyntaxHighlighter as CMakeSyntaxHighlighter, Caches as Caches, CatalogEventHandlers as CatalogEventHandlers, CatalogEventReceiver as CatalogEventReceiver, ClientManager as ClientManager, Color as Color, CppSyntaxHighlighter as CppSyntaxHighlighter, CropWindow as CropWindow, Cursors as Cursors, DrawState as DrawState, Events as Events, ExternalTools as ExternalTools, FCurveUtil as FCurveUtil, FaceSelect as FaceSelect, FileHelpers as FileHelpers, GLContextManager as GLContextManager, GLDrawingRoutines as GLDrawingRoutines, HotRender as HotRender, IconManager as IconManager, KatanaMediaCache as KatanaMediaCache, LayeredMenuUtils as LayeredMenuUtils, LogRecord as LogRecord, LogRecordHandler as LogRecordHandler, LogRecordItemModel as LogRecordItemModel, LuaSyntaxHighlighter as LuaSyntaxHighlighter, ManipulatorManager as ManipulatorManager, NodeDialogs as NodeDialogs, NodeErrors as NodeErrors, NodeGraphTab as NodeGraphTab, NotificationManager as NotificationManager, Number as Number, PluginManager as PluginManager, PythonSyntaxHighlighter as PythonSyntaxHighlighter, PythonWidgetIntrospector as PythonWidgetIntrospector, Record as Record, RecordItemModel as RecordItemModel, RenderMenu as RenderMenu, ScenegraphIconManager as ScenegraphIconManager, ScenegraphLocation as ScenegraphLocation, ScriptButton as ScriptButton, Shader as Shader, TextureManager as TextureManager, TransformManipulatorLayer as TransformManipulatorLayer, UndoGrouping as UndoGrouping, Views as Views, WorkingSets as WorkingSets
from typing import Set, Tuple



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI_cmodule
import PyFnAttribute
import typing
from typing import Any, Set, Tuple, overload

@overload
def BuildAttrListFromDynamicParameterGroup(groupParam: NodegraphAPI_cmodule.Parameter, graphState: NodegraphAPI_cmodule.GraphState, applyLocalSettings: bool = ..., attrPath: str = ..., alwaysIncludeSet: list[str] = ..., multisampleDefault: bool = ...) -> list: ...
@overload
def BuildAttrListFromDynamicParameterGroup(groupParam: NodegraphAPI_cmodule.Parameter, frameTime: float, applyLocalSettings: bool = ..., attrPath: str = ..., alwaysIncludeSet: list[str] = ..., multisampleDefault: bool = ...) -> list: ...
def BuildGroupAttrFromParam(groupParam: NodegraphAPI_cmodule.Parameter, frameTime: float, inherit: bool = ..., returnRawAttrPaths: bool = ..., includeEmpty: bool = ...) -> Any: ...
def BuildHierarchyCreateOpArgs(frameTime: float, groupParameter: NodegraphAPI_cmodule.Parameter, rootAttrName: str = ...) -> PyFnAttribute.GroupAttribute: ...
def DefaultDAPCookOrder() -> PyFnAttribute.Attribute: ...
@overload
def EvaluateBoolExpresion(expression: str, tags: typing.Sequence) -> bool: ...
@overload
def EvaluateBoolExpresion(expression: str, tags: set) -> bool: ...
def GetLeafAttrPairs(groupAttribute: PyFnAttribute.GroupAttribute, prefix: str = ...) -> list: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import AssetAPI as AssetAPI
import ConfigurationAPI_cmodule as Configuration
import Utils.EventModule as EventModule
import NodegraphAPI.GroupStack as GroupStack
import NodegraphAPI.NodeExtensions as NodeExtensions
import NodegraphAPI.NodeXmlIO as NodeXmlIO
import NodegraphAPI as NodegraphAPI
import NodegraphAPI.GroupStack
import NodegraphAPI_cmodule
import NodegraphAPI.NodegraphGlobals as NodegraphGlobals
import PyXmlIO
import PyQt5.QtCore as QtCore
import Utils.UndoStack as UndoStack
import Utils as Utils
import Utils.UndoStack
import NodegraphAPI.Xio as Xio
from Callbacks.Callbacks import Callbacks as Callbacks
from NodegraphAPI_cmodule import GroupNode as GroupNode, PythonGroupNode as PythonGroupNode, node_addNodeFlavor as node_addNodeFlavor, node_getAllNodes as node_getAllNodes, node_registerPythonGroupType as node_registerPythonGroupType
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class AssetPublishing:
    kAssetPublishingCancelled: ClassVar[int] = ...
    kAssetPublishingFailed: ClassVar[int] = ...
    kAssetPublishingStarted: ClassVar[int] = ...
    kAssetPublishingSucceeded: ClassVar[int] = ...
    def __init__(self) -> None: ...
    def errorMessageDialogsEnabled(self) -> bool: ...
    def getAssetType(self) -> str | None: ...
    def getErrorMessage(self) -> str | None: ...
    def getExtraOptions(self) -> dict: ...
    def getFilenameOrAssetIdToPublish(self) -> str | None: ...
    def getNodeName(self) -> str | None: ...
    def getPublishedFilenameOrAssetID(self) -> str | None: ...
    def getState(self) -> int: ...
    def isErrorLoggingEnabled(self) -> bool: ...
    def isOverwriteAssetEnabled(self) -> bool: ...
    def isSuccessLoggingEnabled(self) -> bool: ...
    def setAssetType(self, assetType: str | None): ...
    def setErrorLoggingEnabled(self, errorLoggingEnabled: bool): ...
    def setErrorMessage(self, errorMessage: str | None): ...
    def setErrorMessageDialogsEnabled(self, errorMessageDialogsEnabled: bool): ...
    def setExtraOptions(self, extraOptions: dict): ...
    def setFilenameOrAssetIdToPublish(self, filenameOrAssetIdToPublish: str | None): ...
    def setNodeName(self, nodeName: str | None): ...
    def setOverwriteAssetEnabled(self, overwriteAssetEnabled: bool): ...
    def setPublishedFilenameOrAssetID(self, publishedFilenameOrAssetID: str | None): ...
    def setState(self, state: int): ...
    def setSuccessLoggingEnabled(self, successLoggingEnabled: bool): ...

class LiveGroupMixin:
    EDITABLE_STATES: ClassVar[tuple] = ...
    STATES: ClassVar[tuple] = ...
    STATE_ATTR_NAME: ClassVar[str] = ...
    STATE_CHILD_EDITABLE: ClassVar[int] = ...
    STATE_CHILD_MODIFIED: ClassVar[int] = ...
    STATE_EDITABLE: ClassVar[int] = ...
    STATE_EDITABLE_CHILD_MODIFIED: ClassVar[int] = ...
    STATE_MODIFIED: ClassVar[int] = ...
    STATE_NON_EDITABLE: ClassVar[int] = ...
    _LiveGroupMixin__foundRecursionOnLoading: ClassVar[bool] = ...
    _LiveGroupMixin__loadingRootNode: ClassVar[None] = ...
    _LiveGroupMixin__loadingStack: ClassVar[list] = ...
    def __init__(self) -> None: ...
    def _LiveGroupMixin__adoptPortConfiguration(self, node: NodegraphAPI.Node): ...
    def _LiveGroupMixin__canChangeStateTo(self, editable): ...
    def _LiveGroupMixin__makeEditable(self): ...
    def _LiveGroupMixin__makeNonEditable(self): ...
    def _LiveGroupMixin__mergeParameters(self, loadedGroup): ...
    def _LiveGroupMixin__removeChildren(self): ...
    def _LiveGroupMixin__reparentChildren(self, loadedGroup): ...
    def _isSourcelessLiveGroupModified(self) -> bool: ...
    def _load(self, forceLoad: bool, forceNonEditable: bool = ...): ...
    def _setEditable(self, editable: bool, updateContentLock: bool = ...) -> bool: ...
    def _setLiveGroupState(self, liveGroupState: int): ...
    def _setLiveGroupStateToEditable(self): ...
    def _updateContentLock(self): ...
    def _updateLiveGroupState(self, childLiveGroupNode: LiveGroupNode, previousChildLiveGroupState: int): ...
    def _updateShapeAttributes(self): ...
    def getLastSource(self): ...
    def getNestedLiveGroupNodes(self) -> list: ...
    def getSourceFile(self) -> str: ...
    def getSourceName(self) -> str: ...
    def hasEditableLiveGroupChildren(self) -> bool: ...
    def hasModifiedLiveGroupChildren(self) -> bool: ...
    def isDisabledByParameter(self) -> bool: ...
    def isEditable(self) -> bool: ...
    def setAttributes(self, attrs: dict): ...
    def setSource(self, filenameOrAssetID: str, loadFromSource: bool = ...): ...

class LiveGroupNode(NodegraphAPI_cmodule.PythonGroupNode, LiveGroupMixin):
    ATTRIBUTES_TO_IGNORE: ClassVar[tuple] = ...
    def __init__(self) -> None: ...
    def _LiveGroupNode__resolveLiveGroupReferences(self, documentElement: PyXmlIO.Element) -> tuple[bool, PyXmlIO.Element]: ...
    def _calculateContentHash(self): ...
    def _hasContentChanged(self): ...
    def _sourceAssetIdIsFilePath(self): ...
    def _updateContentHash(self): ...
    def _updateContentLock(self): ...
    def addParameterHints(self, attrName, inputDict): ...
    def convertToGroup(self): ...
    def exportAsset(self, filenameOrAssetID: str, extraOptions: Incomplete | None = ...) -> str: ...
    def getGroupXmlIO(self) -> Document: ...
    def hasContentChanged(self): ...
    def isModified(self) -> bool: ...
    def load(self, lockResult: bool = ..., forceLoad: bool = ..., forceNonEditable: bool = ...): ...
    def makeEditable(self, includingAllParents: bool = ...) -> bool: ...
    def publishAndLoadAsset(self, filenameOrAssetID: Incomplete | None = ..., extraOptions: Incomplete | None = ...): ...
    def publishAsset(self, filenameOrAssetID: Incomplete | None = ..., extraOptions: Incomplete | None = ...) -> str: ...
    def publishAssetAndFinishEditingContents(self, filenameOrAssetID: Incomplete | None = ..., extraOptions: Incomplete | None = ...) -> str: ...
    def reloadFromSource(self) -> bool: ...
    def revert(self) -> bool: ...
    def setAttributes(self, attrs: dict): ...
    def setLocked(self, locked: bool): ...
    def writeToAsset(self, filenameOrAssetID, extraOptions: Incomplete | None = ..., reloadContent: bool = ...): ...

class LiveGroupStack(NodegraphAPI.GroupStack.GroupStackNode, LiveGroupMixin):
    def __init__(self) -> None: ...
    def addParameterHints(self, attrName, inputDict): ...
    def allowChildReparentingViaDrag(self, childNode): ...
    def buildChildNode(self, adoptNode: Incomplete | None = ...): ...
    def canAdoptNodes(self, nodes): ...
    def deleteChildNode(self, childNode): ...
    def load(self, lockResult: bool = ..., forceLoad: bool = ..., forceNonEditable: bool = ...): ...
    def preprocessChildReparentingViaDrag(self, childNode): ...
    def reorderChildNode(self, fromIndex, toIndex): ...

class UndoConvertGroupToLiveGroup(Utils.UndoStack.UndoEntry):
    def __init__(self, groupNodeName, liveGroupNodeName) -> None: ...
    def getName(self) -> str: ...
    def redo(self): ...
    def undo(self): ...

class UndoConvertLiveGroupToGroup(Utils.UndoStack.UndoEntry):
    def __init__(self, groupNodeName, liveGroupNodeName, filenameOrAssetID, isEditable) -> None: ...
    def getName(self) -> str: ...
    def redo(self): ...
    def undo(self): ...

def CalculateLiveGroupDepth(liveGroupNode: LiveGroupMixin | NodegraphAPI.Node) -> int: ...
def CanBeDifferentFromSource(liveGroupNode: LiveGroupMixin) -> bool: ...
def ConvertGroupToLiveGroup(groupNode): ...
def IsLiveGroupCachingEnabled(): ...
def IsLiveGroupLoadingEnabled(): ...
def LoadAllLiveGroups(node: NodegraphAPI.Node, lockResult: bool = ..., forceLoad: bool = ...): ...
def LockAllLiveGroups(node: NodegraphAPI.Node): ...
def MakeAllLiveGroupsEditable(node: NodegraphAPI.Node): ...
def MakeAllParentLiveGroupsEditable(node: NodegraphAPI.Node) -> bool: ...
def SetLiveGroupCachingEnabled(liveGroupCachingEnabled): ...
def SetLiveGroupLoadingEnabled(state): ...
def UpdateAllLiveGroupSources(oldSource: str, newSource: str): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4FormWidgets.HintUtils as HintUtils
import QT4FormWidgets.PaintingUtils as PaintingUtils
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from QT4FormWidgets.BaseValueFormWidget import BaseValueFormWidget as BaseValueFormWidget
from QT4FormWidgets.FormWidget import FormWidget as FormWidget
from _typeshed import Incomplete
from typing import Set, Tuple

class CapsuleFormWidget(BaseValueFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _CapsuleFormWidget__setOptionsFromHints(self, hints, appliedHints: Incomplete | None = ...): ...
    def _CapsuleFormWidget__updateCapsule(self, value): ...
    def _assignValue(self, value): ...
    def _buildControlWidget(self, layout): ...
    def _checkControlWidget(self, enabledItems, oldEnabledItems): ...
    def _interpretValue(self, value): ...
    def _lockChanged(self, state): ...
    def _updateControlWidget(self): ...
    def _updateWithHints(self, originalHints, hints): ...
    def getOptions(self): ...
    def setExclusive(self, exclusive): ...
    def setOptions(self): ...
    def setTransitionValidator(self, validator): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Util.IconManager as IconManager
import PyQt5.QtGui
import PyQt5.QtWidgets
import QT4FormWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4 as UI4
import Utils as Utils
from typing import ClassVar, Set, Tuple

class MonitorManipulatorButton(PyQt5.QtWidgets.QToolButton):
    _icon: ClassVar[None] = ...
    def __init__(self, *args) -> None: ...
    @staticmethod
    def GetIcon() -> PyQt5.QtGui.QIcon: ...
    def _MonitorManipulatorButton__on_clicked(self): ...
    def _MonitorManipulatorButton__on_manipulator_update(self, eventType: str | None, eventID: object, policy: QT4FormWidgets.AbstractValuePolicy, isEnabled: bool): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.Util.IconManager as IconManager
import UI4.NodeMaster.NodeInteractionDelegateManager as NodeInteractionDelegateManager
import NodegraphAPI
import Nodes3DAPI
import PyQt5.QtWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import Utils as Utils
from typing import ClassVar, Set, Tuple

class NodeDropWidget(PyQt5.QtWidgets.QLabel):
    _NodeDropWidget__NODE: ClassVar[str] = ...
    _NodeDropWidget__NODEBIG: ClassVar[str] = ...
    _NodeDropWidget__NODEDRAG: ClassVar[str] = ...
    _NodeDropWidget__NODEDROPONTO: ClassVar[str] = ...
    def __init__(self, parent, node: NodegraphAPI.Node) -> None: ...
    def _NodeDropWidget__startDrag(self): ...
    def customEvent(self, event): ...
    def dragEnterEvent(self, event): ...
    def dragLeaveEvent(self, event): ...
    def dropEvent(self, event): ...
    def getNode(self) -> NodegraphAPI.Node: ...
    def mousePressEvent(self, event): ...
    def setNode(self, node: Nodes3DAPI.Node3D): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
from Nodes3DAPI.Node3D import Node3D as Node3D
from typing import ClassVar, Set, Tuple

class BaseConstraint(Node3D):
    CommonParamXML: ClassVar[str] = ...
    _CommonParamHints: ClassVar[dict] = ...
    def __init__(self) -> None: ...
    def _getOpChain(self, interface): ...
    def addCommonAndNodeParamHints(self, attrName, nodeHints, inputDict): ...
    def addToConstraintList(self, time): ...
    def getAllowMissingTargets(self, time): ...
    def getBasePath(self, time): ...
    def getConstraintArguments(self, time): ...
    def getConstraintType(self, time): ...
    def getPrependFlag(self): ...
    def getRequireTargetBounds(self, time): ...
    def getRespectMotionBlur(self, time): ...
    def getTargetPaths(self, time): ...
    def setRelativeTargets(self, time): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import ConfigurationAPI_cmodule as Configuration
import UI4.Util.ExternalTools as ExternalTools
import NodegraphAPI
import UI4.KatanaPrefs.PrefNames as PrefNames
import PyQt5.QtCore
import QT4FormWidgets as QT4FormWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import Utils as Utils
import typing
from Callbacks.Callbacks import Callbacks as Callbacks
from QT4FormWidgets.TextFormWidget import TextFormWidget
from UI4.Util.CMakeSyntaxHighlighter import CMakeSyntaxHighlighter as CMakeSyntaxHighlighter
from UI4.Util.CppSyntaxHighlighter import CppSyntaxHighlighter as CppSyntaxHighlighter
from UI4.Util.LuaSyntaxHighlighter import LuaSyntaxHighlighter as LuaSyntaxHighlighter
from UI4.Util.PythonSyntaxHighlighter import PythonSyntaxHighlighter as PythonSyntaxHighlighter
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class ExternalEditSessionError(Exception):
    def __init__(self, message, underlyingException: Incomplete | None = ..., processArguments: Incomplete | None = ...) -> None: ...

class ExternalEditSessionRegistry(PyQt5.QtCore.QObject):
    DirectoryRefreshPollingInterval: ClassVar[int] = ...
    externalEditStateChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    sSharedInstance: ClassVar[None] = ...
    def __init__(self, parent: Incomplete | None = ...) -> None: ...
    def _ExternalEditSessionRegistry__addEditSession(self, editSession): ...
    def _ExternalEditSessionRegistry__createTemporaryFileWithContents(self, prefix, suffix, contents) -> str: ...
    def _ExternalEditSessionRegistry__emitStateChangeSignalForEditSession(self, editSession): ...
    def _ExternalEditSessionRegistry__ensureExternalFilesDirectoryExists(self): ...
    def _ExternalEditSessionRegistry__getEditSessionForFilename(self, filename): ...
    def _ExternalEditSessionRegistry__getEditSessionForParameterKey(self, parameterKey): ...
    def _ExternalEditSessionRegistry__getEditSessionsForNode(self, node: NodegraphAPI.Node): ...
    def _ExternalEditSessionRegistry__getExternalFileContentsForEditSession(self, editSession): ...
    def _ExternalEditSessionRegistry__getExternalFilesDirectory(self): ...
    def _ExternalEditSessionRegistry__isMonitoringEditSession(self, editSession): ...
    def _ExternalEditSessionRegistry__launchExternalEditorForSession(self, editSession): ...
    def _ExternalEditSessionRegistry__onSceneLoad(self): ...
    def _ExternalEditSessionRegistry__on_fileSystemWatcher_directoryChanged(self, directoryPath: str): ...
    def _ExternalEditSessionRegistry__on_fileSystemWatcher_fileChanged(self, filePath: str): ...
    def _ExternalEditSessionRegistry__on_node_delete(self, eventType, eventID, node: NodegraphAPI.Node, oldName): ...
    def _ExternalEditSessionRegistry__on_qtimer_timeout(self): ...
    def _ExternalEditSessionRegistry__refreshStaleSessions(self): ...
    def _ExternalEditSessionRegistry__removeEditSession(self, editSession): ...
    def _ExternalEditSessionRegistry__rescanDirectory(self, directoryPath: str): ...
    def _ExternalEditSessionRegistry__setEditSessionIsStale(self, editSession): ...
    def beginEditingParameterExternally(self, parameter, initialValue, tempFilePrefix: Incomplete | None = ..., tempFileSuffix: Incomplete | None = ...): ...
    def endEditingParameterExternally(self, parameter): ...
    def getExternalFileContentsForParameter(self, parameter) -> str | None: ...
    @classmethod
    def getInstance(cls): ...
    def isParameterEditedExternally(self, parameter): ...

class ScriptEditorFormWidget(TextFormWidget):
    def __init__(self, parent, policy, factory) -> None: ...
    def _ScriptEditorFormWidget__endExternalEditSession(self): ...
    def _ScriptEditorFormWidget__getPolicyValue(self): ...
    def _ScriptEditorFormWidget__isBeingEditedExternally(self): ...
    def _ScriptEditorFormWidget__isPolicyValueReadOnly(self): ...
    def _ScriptEditorFormWidget__on_editSessionRegistry_externalEditStateChanged(self, node: NodegraphAPI.Node, parameterName, newValue): ...
    def _ScriptEditorFormWidget__on_externalEditButton_clicked(self): ...
    def _ScriptEditorFormWidget__on_pref_changed(self, eventType: str | None, eventID: typing.Hashable, prefKey: str, prefValue: object): ...
    def _ScriptEditorFormWidget__refreshWidgetState(self): ...
    def _ScriptEditorFormWidget__setExternalEditButtonText(self): ...
    def _ScriptEditorFormWidget__setPolicyValue(self, newValue): ...
    def _ScriptEditorFormWidget__setPolicyValueReadOnly(self, isReadOnly: bool | None) -> bool | None: ...
    def _buildControlWidget(self, layout): ...
    def setLabelWidth(self, width): ...
    def showExternalEditor(self): ...

class _EditSession:
    def __init__(self, parameterKey, filename, fileContents) -> None: ...

class _ParameterKey:
    def __init__(self, node: NodegraphAPI.Node, parameterName) -> None: ...
    @classmethod
    def fromParameter(cls, parameter): ...
    def __eq__(self, other) -> bool: ...
    def __hash__(self) -> int: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import Nodes2DAPI_cmodule as Nodes2DAPI_cmodule
import RenderingAPI as RenderingAPI
import Utils as Utils
import Nodes2DAPI_cmodule.Views as Views
import types
from . import Expression as Expression, FileInNodeUtil as FileInNodeUtil, FileOutNodeUtil as FileOutNodeUtil, GradientNodeUtil as GradientNodeUtil, NodeColorDelegate as NodeColorDelegate, ThumbnailManager as ThumbnailManager, TransformUtil as TransformUtil
from Nodes2DAPI_cmodule import CancelRender as CancelRender, CheckExternalRender as CheckExternalRender, CheckFileIns as CheckFileIns, ClearDisplayWindowCache as ClearDisplayWindowCache, ClearImageHeaderCache as ClearImageHeaderCache, ClearSpeculativeCacheContents as ClearSpeculativeCacheContents, ConvertUpdateRectsToFnAttribute as ConvertUpdateRectsToFnAttribute, CreateExternalRenderListener as CreateExternalRenderListener, EnumerateReadableImageFormats as EnumerateReadableImageFormats, EnumerateWriteableImageFormats as EnumerateWriteableImageFormats, FinishRender as FinishRender, FrameBuffer as FrameBuffer, GetAllRenderTypes as GetAllRenderTypes, GetDisplayWindow as GetDisplayWindow, GetExternalRenderListenerInfo as GetExternalRenderListenerInfo, GetFileOutAssetId as GetFileOutAssetId, GetFileOutColorspace as GetFileOutColorspace, GetFileOutEnabled as GetFileOutEnabled, GetFileOutFilename as GetFileOutFilename, GetFileOutFormat as GetFileOutFormat, GetFileOutOutputParams as GetFileOutOutputParams, GetFileOutPassName as GetFileOutPassName, GetFileOutPostScriptInfo as GetFileOutPostScriptInfo, GetFileOutPostScriptParams as GetFileOutPostScriptParams, GetFileOutProxyOnCue as GetFileOutProxyOnCue, GetFileOutRepPrefix as GetFileOutRepPrefix, GetFileOutRepresentation as GetFileOutRepresentation, GetFileOutResolutionName as GetFileOutResolutionName, GetFilmWidthFromString as GetFilmWidthFromString, GetFrameBufferCancelledMessages as GetFrameBufferCancelledMessages, GetFrameBufferCompletedMessages as GetFrameBufferCompletedMessages, GetFrameBufferRectUpdateMessages as GetFrameBufferRectUpdateMessages, GetImageDescription as GetImageDescription, GetImageDiskUsage as GetImageDiskUsage, GetImageMemoryUsage as GetImageMemoryUsage, GetImageReadPath as GetImageReadPath, GetIncrementedRenderSequenceID as GetIncrementedRenderSequenceID, GetMaxDiskUsage as GetMaxDiskUsage, GetMaxImageMemory as GetMaxImageMemory, GetNewFrameBuffers as GetNewFrameBuffers, GetOutputDisplayWindow as GetOutputDisplayWindow, GetRenderProgressMessages as GetRenderProgressMessages, GetRenderTerminationSignal as GetRenderTerminationSignal, GetRenderThreads as GetRenderThreads, GetSpeculativeCacheContents as GetSpeculativeCacheContents, GetSpeculativeCacheEnabled as GetSpeculativeCacheEnabled, GetTileRenderOrderMessages as GetTileRenderOrderMessages, GetTileSize as GetTileSize, GetTransformBetweenNodes as GetTransformBetweenNodes, GetTransformForInputPort as GetTransformForInputPort, IntImage as IntImage, IsExternalRenderInProgress as IsExternalRenderInProgress, IsRenderInProgress as IsRenderInProgress, IsThumbnailRendering as IsThumbnailRendering, KillExternalRender as KillExternalRender, Node2D as Node2D, NotifyBufferChanged as NotifyBufferChanged, PrintImageMemoryUsage as PrintImageMemoryUsage, QueueErrorMessage as QueueErrorMessage, QueueRenderMessage as QueueRenderMessage, ReadImage as ReadImage, ReadImageThreaded as ReadImageThreaded, RegisterNukeSession as RegisterNukeSession, Render as Render, RenderThumbnail as RenderThumbnail, RestartExternalRender as RestartExternalRender, SetMaxDiskUsage as SetMaxDiskUsage, SetMaxImageMemory as SetMaxImageMemory, SetNukeSessionMappings as SetNukeSessionMappings, SetNukeSessionOutputNodeName as SetNukeSessionOutputNodeName, SetRenderDebug as SetRenderDebug, SetRenderLogFile as SetRenderLogFile, SetRenderThreads as SetRenderThreads, SetSpeculativeCacheEnabled as SetSpeculativeCacheEnabled, SetTileSize as SetTileSize, SignalExternalRender as SignalExternalRender, StartExternalRender as StartExternalRender, TerminateCancelledRenders as TerminateCancelledRenders, TileStitch as TileStitch, UnregisterNukeSession as UnregisterNukeSession, fbo_bind as fbo_bind, fbo_create as fbo_create, fbo_delete as fbo_delete, fbo_init as fbo_init, fbo_unbind as fbo_unbind, glBatchVertex as glBatchVertex, glDrawProjectedLatLongLine as glDrawProjectedLatLongLine, gl_disableFragmentProgram as gl_disableFragmentProgram, gl_enableFilmlookFragmentProgram as gl_enableFilmlookFragmentProgram, gl_enableFragmentProgram as gl_enableFragmentProgram
from _typeshed import Incomplete
from typing import Set, Tuple

RENDER_TYPE_IDLE: int
RENDER_TYPE_PRIMARY: int
TILE_PRIORITIZER_NONE: int
TILE_PRIORITIZER_SCANLINE: int
TILE_PRIORITIZER_SPIRAL: int

def GetErrorMessages(): ...
def GetNewRenders(): ...
def GetRenderMessages(): ...
def GetViewBasedPrioritization(): ...
def IsUpstreamNode2D(node: NodegraphAPI.Node, frame: Incomplete | None = ...): ...
def RegisterCommandLineNode(*args): ...
def RegisterNewRender(renderDict): ...
def Render_Buffer(outputList, sampleRate, ROI, hotRender, interactive, diskCachingAllowed, emitTileRenderOrdering, tilePrioritizer, overscanPadding, renderType, renderMethodName): ...
def ReportRenderMessage(sequenceID, message): ...
def SetCatalogEventReceiver(receiver): ...
def SetFarmPluginManager(farmPluginManager: types.ModuleType): ...
def SetPrioritizerCenter(centerPoint): ...
def SetPrioritizerWindow(window): ...
def SetViewBasedPrioritization(isEnabled): ...
def Set_GetRenderProducer_Fn(function): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.App.AboutKatana as AboutKatana
import AssetAPI as AssetAPI
import CatalogAPI as CatalogAPI
import PyUtilModule.Documentation as Documentation
import UI4.Util.ExternalTools as ExternalTools
import PyUtilModule.FarmAPI as FarmAPI
import PyFnGeolibServices as FnGeolibServices
import UI4.App.Hotkeys as Hotkeys
import UI4.Util.KatanaMediaCache as KatanaMediaCache
import UI4.App.Layouts as Layouts
import PyUtilModule.LiveRenderAPI as LiveRenderAPI
import UI4.App.MainWindow as MainWindow
import NodegraphAPI as NodegraphAPI
import Nodes2DAPI as Nodes2DAPI
import Nodes3DAPI as Nodes3DAPI
import UI4.KatanaPrefs.PrefNames as PrefNames
import UI4.App.Preferences as Preferences
import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import UI4.App.RecentFiles as RecentFiles
import PyUtilModule.RenderManager as RenderManager
import RenderingAPI as RenderingAPI
import UI4.App.Tabs as Tabs
import UI4 as UI4
import Utils as Utils
from UI4.App.KeyboardShortcutManager.KeyboardShortcutManager import GetShortcutForAction as GetShortcutForAction, GetShortcutForKeyEvent as GetShortcutForKeyEvent, RegisterAction as RegisterAction
from UI4.KatanaPrefs.KatanaPrefsObject import Prefs as Prefs
from UI4.Widgets.GetSetMenu import GetSetMenu
from _typeshed import Incomplete
from typing import Set, Tuple

class DevGuideAction(PyQt5.QtWidgets.QAction):
    def __init__(self, text: str, parent: PyQt5.QtCore.QObject, documentPath: str = ...) -> None: ...
    def on_triggered(self): ...

class LayoutsMenu(PyQt5.QtWidgets.QMenu):
    def __init__(self, parent, frame: Incomplete | None = ...) -> None: ...
    def _LayoutsMenu__addActionsForLayouts(self, layoutNames): ...
    def _LayoutsMenu__doAboutToShow(self): ...
    def _LayoutsMenu__onActionHovered(self, action): ...
    def doEditLayouts(self): ...
    def doLayoutChange(self, events): ...
    def doSaveLayout(self): ...
    def doSelectLayout(self): ...
    def doShowMainTimeline(self, shown): ...
    def leaveEvent(self, event: PyQt5.QtCore.QEvent): ...
    def populate(self): ...

class MainMenu(PyQt5.QtWidgets.QMenuBar):
    def __init__(self, parent: PyQt5.QtCore.QObject) -> None: ...
    def _MainMenu__getBoundKey(self, key: PyQt5.QtGui.QKeySequence.StandardKey, default: str | PyQt5.QtCore.Key | PyQt5.QtGui.QKeySequence | list | tuple) -> PyQt5.QtGui.QKeySequence.StandardKey | str | PyQt5.QtCore.Key | PyQt5.QtGui.QKeySequence | list | tuple: ...
    def _MainMenu__on_editMenu_aboutToHide(self): ...
    def _MainMenu__on_editMenu_aboutToShow(self): ...
    def _MainMenu__populate(self): ...

class OnlineHelpAction(PyQt5.QtWidgets.QAction):
    def __init__(self, text: str, parent: PyQt5.QtCore.QObject, cshid: Incomplete | None = ...) -> None: ...
    def on_triggered(self): ...

class OpenRecentAction(PyQt5.QtWidgets.QAction):
    def __init__(self, label, parent, meta) -> None: ...

class OpenRecentMenu(PyQt5.QtWidgets.QMenu):
    def __init__(self, parent) -> None: ...
    def _OpenRecentMenu__doAboutToShow(self): ...
    def _OpenRecentMenu__doTriggered(self, action): ...

class RenderMenu(GetSetMenu):
    def __init__(self, parent, frame: Incomplete | None = ...) -> None: ...
    def _RenderMenu__cancelAllRenders(self): ...
    def _RenderMenu__cancelCurrentRender(self): ...
    def _RenderMenu__liveRenderViewNode(self): ...
    def _RenderMenu__on_aboutToShow(self): ...
    def _RenderMenu__on_showKatanaQueueAction_triggered(self): ...
    def _RenderMenu__on_stopRenderFarmJobsAction_triggered(self): ...
    def _RenderMenu__previewRenderViewNode(self): ...
    def _RenderMenu__render2DMenuAboutToShow(self): ...
    def _RenderMenu__repeatPreviousRender(self): ...
    def _RenderMenu__stopAllRenderFarmJobs(self, farmPluginName: str): ...
    def _RenderMenu__viewBasedPrioritizationAction_CB(self): ...

class Rendering2DMenu(PyQt5.QtWidgets.QMenu):
    def __init__(self, *args) -> None: ...
    def _Rendering2DMenu__aboutToShow(self): ...
    def _Rendering2DMenu__buildActions(self): ...
    def _Rendering2DMenu__printImageMemoryUsage(self): ...
    def _Rendering2DMenu__resetRenderThreadPool(self): ...
    def _Rendering2DMenu__setRenderThreadsPref(self, value): ...
    def _Rendering2DMenu__viewBasedPrioritizationAction_CB(self): ...

class Rendering3DMenu(PyQt5.QtWidgets.QMenu):
    def __init__(self, *args) -> None: ...
    def _Rendering3DMenu__aboutToShow(self): ...
    def _Rendering3DMenu__buildActions(self): ...
    def _Rendering3DMenu__pauseRender(self): ...
    def _Rendering3DMenu__queryRender(self): ...
    def _Rendering3DMenu__renderIDPass(self): ...
    def _Rendering3DMenu__resumeRender(self): ...

class RenderingLiveRenderMenu(PyQt5.QtWidgets.QMenu):
    def __init__(self, title, activateShortcuts: bool = ..., parent: Incomplete | None = ...) -> None: ...
    def _RenderingLiveRenderMenu__buildActions(self, activateShortcuts: bool = ...): ...
    def _RenderingLiveRenderMenu__on_aboutToShow(self): ...

class TabMenuItem(PyQt5.QtWidgets.QAction):
    def __init__(self, displayName, tabTypeName, parent) -> None: ...

class TabsMenu(PyQt5.QtWidgets.QMenu):
    def __init__(self, parent, frame: Incomplete | None = ...) -> None: ...
    def _TabsMenu__addTabMenuItems(self, tabTypeNames: list[str]): ...
    def _TabsMenu__doAboutToShow(self): ...
    def _TabsMenu__doKatanaPanel(self, args): ...
    def on_triggered(self, action): ...
    def populate(self): ...

class UtilMenu(PyQt5.QtWidgets.QMenu):
    def __init__(self, parent) -> None: ...
    def _UtilMenu__doAboutToShow(self): ...
    def _UtilMenu__executeFarmMenuOptionCallback(self, callback: Incomplete | None = ...): ...
    def _UtilMenu__farmXML(self): ...
    def _UtilMenu__flushCaches(self): ...
    def _UtilMenu__refreshLocalMediaCache(self): ...
    def _UtilMenu__reloadArgsFiles(self): ...
    def _UtilMenu__reloadShadersAndArgsFiles(self): ...
    def _UtilMenu__resetAssetAPIPlugins(self, eventType, eventID, **kwargs): ...
    def _UtilMenu__syncOutputPorts(self): ...

class VersionUpAndSaveAction(PyQt5.QtWidgets.QAction):
    def __init__(self, parent) -> None: ...
    def _VersionUpAndSaveAction__checkEnabled(self, *args, **kwargs): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import CatalogWidget as CatalogWidget, CatalogWidgetItem as CatalogWidgetItem, ImageImportDialog as ImageImportDialog
from UI4.Tabs.Catalog.CatalogPanel import CatalogPanel as CatalogPanel
from typing import Set, Tuple

PluginRegistry: list



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import Nodes2DAPI as Nodes2DAPI
import PyQt5.QtWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtWidgets as QtWidgets
import PyUtilModule.RenderManager as RenderManager
import UI4 as UI4
import Utils as Utils
from QT4Widgets.CapsuleCombo import CapsuleCombo
from typing import Set, Tuple

class RenderViewCapsule(CapsuleCombo):
    def __init__(self, parent) -> None: ...
    def _RenderViewCapsule__update_CB(self, *args, **kwargs): ...
    def _RenderViewCapsule__valueChanged_CB(self, newEnabledItems, oldEnabledItems): ...
    def updateState(self): ...

class RenderViewsAutoUpdateCheckbox(PyQt5.QtWidgets.QCheckBox):
    def __init__(self, displayName, parent) -> None: ...
    def _RenderViewsAutoUpdateCheckbox__clicked_CB(self, checked): ...
    def _RenderViewsAutoUpdateCheckbox__update_CB(self, *args, **kwargs): ...
    def updateState(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtWidgets
import QT4FormWidgets as QT4FormWidgets
import QT4FormWidgets.ValuePolicy
import PyQt5.QtCore as QtCore
from QT4FormWidgets.FilterablePopupFormWidget import FilterablePopupFormWidget
from QT4FormWidgets.WidgetFactory import WidgetFactory
from UI4.Widgets.LookFileMaterialComboBox import LookFileMaterialComboBox as LookFileMaterialComboBox
from typing import Set, Tuple

class LookFileMaterialPopupFormWidget(FilterablePopupFormWidget):
    def __init__(self, parent: PyQt5.QtWidgets.QWidget, policy: QT4FormWidgets.ValuePolicy.AbstractValuePolicy, factory: WidgetFactory) -> None: ...
    def _buildControlWidget(self, layout): ...
    def _buildLabel(self, labelText, pos: int = ...): ...
    def _lockChanged(self, state): ...
    def _updateControlWidget(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import CatalogAPI as CatalogAPI
import Nodes2DAPI as Nodes2DAPI
import QT4GLLayerStack as QT4GLLayerStack
import QT4GLLayerStack.LayerStack
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import UI4 as UI4
from PyUtilModule.VirtualKatana import DrawState as DrawState
from typing import Set, Tuple

class HotkeyLayer(QT4GLLayerStack.LayerStack.Layer):
    def __init__(self, *args, **kwargs) -> None: ...
    def processEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import PyQt5.QtCore
import PyQt5.QtCore as QtCore
import UI4.Widgets.TreeWidget
from PyUtilModule.VirtualKatana import Widgets as Widgets
from UI4.Widgets.TreeWidget import TreeWidget
from _typeshed import Incomplete
from typing import ClassVar, Set, Tuple

class DopeSheetItem(UI4.Widgets.TreeWidget.TreeItem):
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def Prune(cls, child, keep: Incomplete | None = ...): ...
    def _KeyManagingItem__addKeys(self, keys): ...
    def _setKeysFromChildren(self): ...
    def _setKeysFromChildrenSelected(self): ...
    def _setKeysSelectedAndUpdate(self, keys, selected, updateParent: bool = ...): ...
    def addChild(self, child, **kwargs): ...
    def buildContextMenu(self, menu): ...
    def clear(self): ...
    def clearKeys(self): ...
    def clearKeysSelected(self): ...
    def getDescendant(self, childNameList, create: bool = ...): ...
    def getFullNameBelow(self, excludedParent, separator: str = ...): ...
    def getKeys(self): ...
    def getKeysSelected(self): ...
    def hasKeys(self): ...
    def hasKeysSelected(self): ...
    def iterKeys(self): ...
    def iterKeysSelected(self): ...
    def iterKeysUnselected(self): ...
    def setKeys(self, keys): ...
    def setKeysSelected(self, keys, selected): ...
    def takeChild(self, index): ...
    def takeChildren(self): ...
    def toggleKeySelected(self, key): ...

class DopeSheetTreeWidget(TreeWidget):
    contextMenu: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def _buildRootItem(self, parent, name): ...
    def buildItemContextMenu(self, item, menu): ...
    def resizeEvent(self, event): ...

class KeyManagingItem(UI4.Widgets.TreeWidget.TreeItem):
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def Prune(cls, child, keep: Incomplete | None = ...): ...
    def _KeyManagingItem__addKeys(self, keys): ...
    def _setKeysFromChildren(self): ...
    def _setKeysFromChildrenSelected(self): ...
    def _setKeysSelectedAndUpdate(self, keys, selected, updateParent: bool = ...): ...
    def addChild(self, child, **kwargs): ...
    def buildContextMenu(self, menu): ...
    def clear(self): ...
    def clearKeys(self): ...
    def clearKeysSelected(self): ...
    def getDescendant(self, childNameList, create: bool = ...): ...
    def getFullNameBelow(self, excludedParent, separator: str = ...): ...
    def getKeys(self): ...
    def getKeysSelected(self): ...
    def hasKeys(self): ...
    def hasKeysSelected(self): ...
    def iterKeys(self): ...
    def iterKeysSelected(self): ...
    def iterKeysUnselected(self): ...
    def setKeys(self, keys): ...
    def setKeysSelected(self, keys, selected): ...
    def takeChild(self, index): ...
    def takeChildren(self): ...
    def toggleKeySelected(self, key): ...

class SelectionMatchingItem(KeyManagingItem):
    def _setKeysFromChildrenSelected(self): ...
    def _setKeysSelectedAndUpdate(self, keys, selected, **kwargs): ...
    def selected(self): ...
    def setSelected(self, selected, **kwargs): ...

def GetItemFromParamName(tree, paramFullName, create: bool = ...): ...
def GetParamFromItem(item, paramRoot: Incomplete | None = ...): ...
def GetParamNameFromItem(item, paramRoot: Incomplete | None = ...): ...
def PruneParam(item, paramRoot: Incomplete | None = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import PyFnAttribute
import PyQt5.QtWidgets
import PyQt5.QtGui as QtGui
import UI4.Widgets.SceneGraphView.ColumnPresetManager
import UI4.Widgets.SceneGraphView.Filtering
import UI4.Widgets.SceneGraphView.SceneGraphViewColumn
import Utils as Utils
import typing
from UI4.Widgets.SceneGraphView.BridgeImpl import BridgeImpl as BridgeImpl
from UI4.Widgets.SceneGraphView.ColumnDataType import ColumnDataType as ColumnDataType
from UI4.Widgets.SceneGraphView.Filtering.RuleConfigDialog import RuleConfigDialog as RuleConfigDialog
from UI4.Widgets.SceneGraphView.SceneGraphHandle import SceneGraphHandle as SceneGraphHandle
from UI4.Widgets.SceneGraphView.SceneGraphViewColumn import SceneGraphColumnTitle as SceneGraphColumnTitle
from UI4.Widgets.SceneGraphView.TreeWidgetViewLink import TreeWidgetViewLink as TreeWidgetViewLink
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import Set, Tuple

class SceneGraphView:
    def __init__(self, debugMode: bool = ..., parent: Incomplete | None = ...) -> None: ...
    @classmethod
    def GetStopAtLocationTypes(cls) -> tuple[str, ...]: ...
    @classmethod
    def Initialize(cls): ...
    @classmethod
    def SetStopAtLocationTypes(cls, *locationTypes): ...
    def addAttributeDataItemType(self, itemTypeClass: type): ...
    def addColumn(self, columnName: str) -> SceneGraphColumn[UI4.Widgets.SceneGraphView.SceneGraphViewColumn.SceneGraphColumn]: ...
    def addColumnSet(self, columnSetName: str) -> SceneGraphColumnSet[UI4.Widgets.SceneGraphView.SceneGraphViewColumn.SceneGraphColumnSet]: ...
    def addRule(self, ruleName: str): ...
    def addTopLevelLocation(self, topLevelLocationPath: str, index: Incomplete | None = ...): ...
    def addWorkingSetColumn(self, workingSetName: str, columnIcon: Incomplete | None = ..., columnTitle: Incomplete | None = ..., columnTitleToolTip: Incomplete | None = ...) -> SceneGraphColumn[UI4.Widgets.SceneGraphView.SceneGraphViewColumn.SceneGraphColumn]: ...
    def allowMultipleSelection(self) -> bool | None: ...
    def applyColumnVisibilitySettings(self, columnVisibility: dict): ...
    def applyColumnWidthsSettings(self, columnWidths: dict): ...
    def beginColumnConfiguration(self): ...
    def clearExpandedLocationsRegistry(self, clearCurrent: bool = ...): ...
    def clearTopLevelLocations(self): ...
    def endColumnConfiguration(self): ...
    def frozenWhenHidden(self) -> bool | None: ...
    def getAllLocations(self, visibleOnly: bool = ...) -> list[tuple | None]: ...
    def getChildLocations(self, locationPath: str, topLevelLocationPath: str | None, visibleOnly: bool = ..., allDescendants: bool = ...) -> list[str | None]: ...
    def getChildNameByIndex(self, parentLocationPath, childIndex): ...
    def getColumnByName(self, columnName: str) -> SceneGraphColumn[UI4.Widgets.SceneGraphView.SceneGraphViewColumn.SceneGraphColumn] | None: ...
    def getColumnPresetManager(self) -> ColumnPresetManager[UI4.Widgets.SceneGraphView.ColumnPresetManager.ColumnPresetManager]: ...
    def getColumnVisibilitySettings(self) -> dict: ...
    def getColumnWidthsSettings(self) -> dict: ...
    def getCurrentRules(self): ...
    def getNumberOfChildren(self, parentLocationPath): ...
    def getParent(self, locationPath: str) -> str | None: ...
    def getRootColumnSet(self) -> SceneGraphColumnSet[UI4.Widgets.SceneGraphView.SceneGraphViewColumn.SceneGraphColumnSet]: ...
    def getSceneGraphAttributes(self, locationPath: str, callback: Incomplete | None = ..., oneShot: bool = ...) -> PyFnAttribute.GroupAttribute | None: ...
    def getSceneGraphChildren(self, parentLocationPath: str, callback: Incomplete | None = ..., oneShot: bool = ...) -> list | None: ...
    def getSelectedItems(self) -> list[TreeWidgetItem | None]: ...
    def getSelectedLocations(self, sortBySceneGraphOrder: bool = ...) -> list[tuple | None]: ...
    def getTopLevelLocations(self) -> list[str]: ...
    def getWidget(self) -> PyQt5.QtWidgets.QWidget | None: ...
    def hideColumnSet(self, columnSetName: str): ...
    def interruptClientProcessing(self): ...
    def isFrozen(self) -> bool: ...
    def isLocationExpanded(self, topLevelLocationPath: str, locationPath: str) -> bool | None: ...
    def isProcessing(self) -> bool: ...
    def isStopping(self) -> bool: ...
    def locationNamesPolished(self) -> bool | None: ...
    def removeRule(self, ruleName: str): ...
    def removeTopLevelLocation(self, topLevelLocationPath: str): ...
    def saveExpandedLocations(self): ...
    def scrollToLocation(self, topLevelLocationPath: str, locationPath: str): ...
    def selectChildLocations(self, replaceSelection: bool = ...): ...
    def selectLocations(self, locations, replaceSelection: bool = ...): ...
    def selectParentLocations(self, replaceSelection: bool = ...): ...
    def setAboutToDragCallback(self, callback: typing.Callable): ...
    def setAllowMultipleSelection(self, allowMultipleSelection: bool): ...
    def setContextMenuEventCallback(self, callback): ...
    def setDragMoveEventCallback(self, callback: typing.Callable): ...
    def setDropEventCallback(self, callback: typing.Callable): ...
    def setExpandsOnDoubleClick(self, expandsOnDoubleClick: bool): ...
    def setFrozenWhenHidden(self, frozenWhenHidden: bool = ...): ...
    def setKeyPressEventCallback(self, callback: typing.Callable): ...
    def setLocationActive(self, locationPath: str): ...
    def setLocationAddedOrUpdatedCallback(self, callback: typing.Callable): ...
    def setLocationCollapsed(self, topLevelLocationPath: str, locationPath: str): ...
    def setLocationCollapsedCallback(self, callback: typing.Callable): ...
    def setLocationDataProcessedCallback(self, callback: typing.Callable): ...
    def setLocationExpanded(self, topLevelLocationPath: str, locationPath: str): ...
    def setLocationExpandedCallback(self, callback: typing.Callable): ...
    def setLocationExpandedRecursive(self, topLevelLocationPath: str, locationPath: str): ...
    def setLocationNamePolishCallback(self, callback: typing.Callable): ...
    def setLocationNamesPolished(self, namesPolished: bool): ...
    def setLocationRemovedCallback(self, callback: typing.Callable): ...
    def setMaxEventsToProcess(self, maxEventsToProcess: int): ...
    def setOverrideParameterRequestCallback(self, callback: typing.Callable): ...
    def setRuleCriteria(self, ruleName: str, criteriaName: str, criteria: Criteria[UI4.Widgets.SceneGraphView.Filtering.Criteria]): ...
    def setRuleEvaluator(self, ruleName: str, evaluator: Evaluator[UI4.Widgets.SceneGraphView.Filtering.Evaluator]): ...
    def setRuleMatchPolicy(self, ruleMatchPolicy: int): ...
    def setRuleTarget(self, ruleName: str, column: SceneGraphColumn[UI4.Widgets.SceneGraphView.SceneGraphViewColumn.SceneGraphColumn]): ...
    def setSelectionChangedCallback(self, callback: typing.Callable): ...
    def setSelectionState(self, topLevelLocationPath: str, locationPath: str, selected: bool): ...
    def setStopRecursiveExpandAttributeNames(self, attributeNames: list[str]): ...
    def setStopRecursiveExpandLocationTypes(self, locationTypes: list[str]): ...
    def setTerminalOps(self, terminalOpArgs): ...
    def setTopLevelDisplayIndex(self, locationPath: str, index: int): ...
    def setTopLevelLocations(self, topLevelLocationPaths: list[str]): ...
    def setUpdateViewportAutomatically(self, updateViewportAutomatically: bool): ...
    def setViewNode(self, node: NodegraphAPI.Node | None, applyImplicitResolvers: bool = ..., extraImplicitResolverOpArgs: Incomplete | None = ...): ...
    def setViewOp(self, op: FnGeolibOp | None, applyImplicitResolvers: bool = ..., extraImplicitResolverOpArgs: Incomplete | None = ...): ...
    def showColumnSet(self, columnSetName: str): ...
    def showRuleConfigDialog(self): ...
    def topLevelLocationRenamed(self, oldLocationPath, newLocationPath): ...
    def updateLocation(self, topLevelLocationPath: str, locationPath: str): ...
    def updateLocationIcons(self, topLevelLocationPath: str, locationPath: str): ...
    def updateSelection(self, selectedLocations, deselectedLocations): ...
    def updateTerminalOps(self, terminalOpArgs, txn: Incomplete | None = ...): ...
    def updateViewport(self): ...
    def updateWidget(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtWidgets
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtWidgets as QtWidgets
import QT4Widgets.WidgetUtils as WidgetUtils
from UI4.Widgets.ToolbarButton import ToolbarButton as ToolbarButton
from typing import ClassVar, Set, Tuple

class HidingFrame(PyQt5.QtWidgets.QFrame):
    hideSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    showSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def hideEvent(self, e): ...
    def showEvent(self, e): ...

class PopupButton(ToolbarButton):
    aboutToShow: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, *args, **kwds) -> None: ...
    def _PopupButton__buildPopup(self): ...
    def _PopupButton__mousePressEvent(self, event): ...
    def _PopupButton__popupHidden(self): ...
    def _PopupButton__popupShow(self): ...
    def getPopup(self): ...
    def setDesiredHeight(self, h): ...
    def setDesiredWidth(self, w): ...
    def setHorizontallyResizable(self, horizontallyResizable): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from typing import Set, Tuple

class Marker:
    def __init__(self, markerText) -> None: ...
    def __del__(self) -> None: ...

def begin(additionalText: str = ...): ...
def end(): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4GLLayerStack as QT4GLLayerStack
import QT4GLLayerStack.LayerStack
import QT4Widgets as QT4Widgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import UI4 as UI4
from UI4.Tabs.Monitor.Layers.ImageLayerBase import ImageLayerBase as ImageLayerBase
from UI4.Util.GLDrawingRoutines import drawEllipse as drawEllipse, drawSmoothArrow as drawSmoothArrow, drawSmoothCross as drawSmoothCross, drawSmoothCrosshairs as drawSmoothCrosshairs, drawSmoothEllipse as drawSmoothEllipse, drawSmoothHandle as drawSmoothHandle, drawSmoothPoly as drawSmoothPoly, drawSmoothText as drawSmoothText
from typing import Set, Tuple

class LinearSwipeLayer(QT4GLLayerStack.LayerStack.Layer, ImageLayerBase):
    def __init__(self, *args, **kwargs) -> None: ...
    def defaultProcessEvent(self, event): ...
    def drawHandles(self, pick: bool = ...): ...
    def getViewIndexFromWindowPos(self, qpos): ...
    def getWindowEndPos(self, posAxis: bool = ...): ...
    def paintDropArea(self, dropColor): ...
    def paintGL(self): ...
    def pickHandles(self, x, y): ...
    def processDragHandleEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

from . import FaceSelectionHelper as FaceSelectionHelper, Layers as Layers, UVWidget as UVWidget
from UI4.Tabs.UVPanel.UVPanel import UVPanel as UVPanel
from typing import Set, Tuple

PluginRegistry: list



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
import NodegraphAPI_cmodule as NodegraphAPI_cmodule
import NodegraphAPI.StructuredPorts as StructuredPorts
import Utils as Utils
from _typeshed import Incomplete
from typing import Set, Tuple

def BakeParameterToFCurve(param, times): ...
def CollapseToGroup(nodes): ...
def DeleteAllNonContributingNodes(outputNodes: Incomplete | None = ..., preserveBackdropNodes: bool = ..., **kwargs): ...
def DuplicateNodes(nodes, keepConnections: bool = ...): ...
def GenericParameterReset(node: NodegraphAPI.Node): ...
def GetAllConnectedInputs(sourceNodes, asList: bool = ...): ...
def GetAllConnectedOutputs(sourceNodes, returnArray: Incomplete | None = ...): ...
def GetAllContributingNodes(outputNodes: Incomplete | None = ...): ...
def GetDescendants(nodes): ...
def GetExpressionReference(nodeName, paramName): ...
def GetExpressionedDuplicates(sourceNodes, deepClone: bool = ...): ...
def IsSpecifiedNodeTypeAnAncestor(node: NodegraphAPI.Node, nodeTypes: list[str]) -> bool: ...
def IsSpecifiedNodeTypeUpstream(node: NodegraphAPI.Node, nodeTypes, checkedNodes: Incomplete | None = ...): ...
def IsolateNode(node: NodegraphAPI.Node): ...
def ParameterReset(node: NodegraphAPI.Node): ...
def ReorderInputPorts(node: NodegraphAPI.Node, oldPos, newPos): ...
def ReorderOutputPorts(node: NodegraphAPI.Node, oldPos, newPos): ...
def WriteCurveXML(curve): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import ConditionalStateGrammar as ConditionalStateGrammar
import Nodes3DAPI.DynamicParameterUtil as DynamicParameterUtil
import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import NodegraphAPI as NodegraphAPI
import PyFnAttribute
import QT4FormWidgets
import Utils as Utils
import typing
from ConditionalStateGrammar.Parser import ParseCSG as ParseCSG
from Nodes3DAPI.Node3D import Node3D as Node3D
from Nodes3DAPI_cmodule import BuildAttrListFromDynamicParameterGroup as BuildAttrListFromDynamicParameterGroup
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import Set, Tuple

_ParamHints: dict
_Parameter_XML: str

class ConstraintEdit(Node3D):
    def __init__(self) -> None: ...
    def _ConstraintEdit__addAttributesToSet(self, graphState: NodegraphAPI.GraphState, argsParam, argsGb): ...
    def _ConstraintEdit__getConstraintIsNewParamValue(self): ...
    def _ConstraintEdit__getSourceOp(self, graphState: NodegraphAPI.GraphState, transaction): ...
    def _ConstraintEdit__setConstraintIsNewParam(self, constraintIsNew): ...
    def _ConstraintEdit__setConstraintParam(self, constraintAttr: PyFnAttribute.GroupAttribute): ...
    def _getIncomingSceneOpAndLocation(self, port: NodegraphAPI.Port, graphState: NodegraphAPI.GraphState, transaction): ...
    def _getOpChain(self, interface): ...
    def _getStaticAttrHintsForIncomingSceneQuery(self, attrPath: str) -> dict: ...
    def _updateParameters(self, groupAttr, force: bool = ..., defaultAttr: Incomplete | None = ...): ...
    def addParameterHints(self, attrName, inputDict): ...
    def callNewPolicyCreatedCallback(self, parameterPolicy: QT4FormWidgets.ParameterPolicy): ...
    def getScenegraphLocation(self, frameTime: Incomplete | None = ...): ...
    def getScenegraphLocationPath(self, time): ...
    def setNewPolicyCreatedCallback(self, newPolicyCreatedCallback: typing.Callable): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import QT4Widgets.Timeline as Timeline
from typing import ClassVar, Set, Tuple

class FrameButtonWidget(PyQt5.QtWidgets.QToolButton):
    def __init__(self, text, parent) -> None: ...

class FrameEditWidget(PyQt5.QtWidgets.QLineEdit):
    def __init__(self, parent, callback) -> None: ...
    def changeEvent(self, event): ...

class FramesPopup(PyQt5.QtWidgets.QFrame):
    closed: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent) -> None: ...
    def addRow(self): ...
    def addWidget(self, text, widget): ...
    def closeEvent(self, event): ...
    def popup(self, pos): ...

class TimebarWidget(PyQt5.QtWidgets.QFrame):
    currentTimeChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    fullRangeChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    incrementChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    selectionChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    timeRangeChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, *args) -> None: ...
    def _TimebarWidget__addLabeledEdit(self, labelText, callback, layout): ...
    def _TimebarWidget__backButton_clicked_callback(self): ...
    def _TimebarWidget__backKeyButton_clicked_callback(self): ...
    def _TimebarWidget__buildChildren(self): ...
    def _TimebarWidget__doEditCurrent(self): ...
    def _TimebarWidget__doEditEnd(self): ...
    def _TimebarWidget__doEditIncrement(self): ...
    def _TimebarWidget__doEditStart(self): ...
    def _TimebarWidget__emitCurrentTimeChanged(self, time, final): ...
    def _TimebarWidget__forwardButton_clicked_callback(self): ...
    def _TimebarWidget__forwardKeyButton_clicked_callback(self): ...
    def _TimebarWidget__resetButton_clicked_callback(self): ...
    def _TimebarWidget__timeline_currentTimeChanged_CB(self, time, final): ...
    def _TimebarWidget__timeline_fullRangeChanged(self, inTime, outTime): ...
    def _TimebarWidget__timeline_selectionChanged(self): ...
    def _TimebarWidget__timeline_timeRangeChanged(self, inTime, outTime): ...
    def addKey(self, time): ...
    def backButton(self): ...
    def backKey(self): ...
    def backKeyButton(self): ...
    def backStep(self): ...
    def clearKeys(self): ...
    def currentTime(self): ...
    def currentTimeEdit(self): ...
    def endTimeEdit(self): ...
    def forwardButton(self): ...
    def forwardKey(self): ...
    def forwardKeyButton(self): ...
    def forwardStep(self): ...
    def fullRange(self): ...
    def increment(self): ...
    def incrementEdit(self): ...
    def isFullRangeFinal(self): ...
    def isTimeRangeFinal(self): ...
    def keySet(self): ...
    def removeKey(self, time): ...
    def resetButton(self): ...
    def selectedTimeRange(self): ...
    def setCurrentTime(self, time): ...
    def setFullRange(self, range): ...
    def setIncrement(self, i): ...
    def setKeys(self, keys): ...
    def setSelectedTimeRange(self, range): ...
    def setTimeRange(self, range): ...
    def startTimeEdit(self): ...
    def timeRange(self): ...
    def timeline(self): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import NodegraphAPI as NodegraphAPI
from _typeshed import Incomplete
from typing import Set, Tuple

def SetTransformFromMatrix(transformGroup, matrix, time: Incomplete | None = ..., final: bool = ..., setKey: bool = ..., pivot: Incomplete | None = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import UI4.FormMaster.Editors.NodeGroup as NodeGroup
import PyFnAttribute as PyFnAttribute
import UI4 as UI4
import UI4.FormMaster.Editors.NodeGroup
import Utils as Utils
from typing import Set, Tuple

class AttributeSetEditor(UI4.FormMaster.Editors.NodeGroup.NodeGroupFormWidget):
    def __init__(self, parent, valuePolicy, widgetFactory) -> None: ...
    def _AttributeSetEditor__attrDropped(self, attr, name): ...

def _GetAttrTypeString(attr): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import ConfigurationAPI_cmodule as Configuration
import drawing_cmodule as DrawingModule
import NodegraphAPI as NodegraphAPI
import Utils as Utils
from Utils.Decorators import deprecated as deprecated
from _typeshed import Incomplete
from typing import Set, Tuple

def AutoPositionNodes(nodes, oldStyle: bool = ...): ...
def FitBackdropNodeAroundNodes(nodes: list, backdropNode: Incomplete | None = ..., padding: int = ...): ...
def FitStickyAroundNodes(nodes, sticky: Incomplete | None = ..., padding: int = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import KatanaResources as KatanaResources
import PyQt5.QtGui
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import Utils as Utils
import typing
from typing import Set, Tuple

def GetIcon(iconName: str) -> PyQt5.QtGui.QIcon | None: ...
def GetPixmap(iconName: str, resolution: int = ..., mode: PyQt5.QtGui.QIcon.Mode = ..., state: PyQt5.QtGui.QIcon.State = ...) -> PyQt5.QtGui.QPixmap: ...
def RegisterIconOverrideCallback(callback: typing.Callable): ...
def UnregisterIconOverrideCallback(callback: typing.Callable): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import AssetAPI_cmodule as AssetAPI_cmodule
import KatanaResources as KatanaResources
import Utils as Utils
from AssetAPI_cmodule import AssetPlugin as AssetPlugin, AssetTransaction as AssetTransaction, FileSequence as FileSequence, FileSequencePlugin as FileSequencePlugin, GetAssetPlugin as GetAssetPlugin, GetAssetPluginNames as GetAssetPluginNames, GetDefaultAssetPlugin as GetDefaultAssetPlugin, GetDefaultAssetPluginName as GetDefaultAssetPluginName, GetDefaultFileSequencePlugin as GetDefaultFileSequencePlugin, GetDefaultFileSequencePluginName as GetDefaultFileSequencePluginName, GetFileSequencePlugin as GetFileSequencePlugin, GetFileSequencePluginNames as GetFileSequencePluginNames, ResetAllAssetAPIPlugins as ResetAllAssetAPIPlugins, SetDefaultAssetPluginName as SetDefaultAssetPluginName, SetDefaultFileSequencePluginName as SetDefaultFileSequencePluginName
from typing import Set, Tuple

kAssetContextAlembic: str
kAssetContextAttributeFile: str
kAssetContextCastingSheet: str
kAssetContextCatalog: str
kAssetContextFCurveFile: str
kAssetContextFarm: str
kAssetContextGafferThreeRig: str
kAssetContextImage: str
kAssetContextKatanaScene: str
kAssetContextLiveGroup: str
kAssetContextLookFile: str
kAssetContextLookFileMgrSettings: str
kAssetContextMacro: str
kAssetContextScenegraphBookmarks: str
kAssetContextShader: str
kAssetFieldName: str
kAssetFieldVersion: str
kAssetRelationArgsFile: str
kAssetTypeAlembic: str
kAssetTypeAttributeFile: str
kAssetTypeCastingSheet: str
kAssetTypeFCurveFile: str
kAssetTypeGafferThreeRig: str
kAssetTypeImage: str
kAssetTypeKatanaScene: str
kAssetTypeLiveGroup: str
kAssetTypeLookFile: str
kAssetTypeLookFileMgrSettings: str
kAssetTypeMacro: str
kAssetTypeScenegraphBookmarks: str
kAssetTypeShader: str
kFnAssetCreationOptionOutputFormat: str
kFnAssetOutputFormatArchive: str
kFnAssetOutputFormatDirectory: str

def ResolvePath(assetPath, frameTime): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import NodegraphAPI as NodegraphAPI
import Nodes3DAPI_cmodule as Nodes3DAPI_cmodule
import PyFnAttribute
import PyXmlIO as PyXmlIO
from _typeshed import Incomplete
from typing import Set, Tuple

def BuildParameterXmlIOFromAttrs(inputParameter, attrList, includeDefault: bool = ..., forcePersistant: bool = ..., dynamicArrayAttrNames: Incomplete | None = ...): ...
def GetAttrNamesFromPaths(attrPaths, prefix): ...
def GetDynamicArrayAttrNames(groupAttr: PyFnAttribute.GroupAttribute, prefix: str) -> list[str]: ...
def GetDynamicArrayAttrPaths(groupAttr: PyFnAttribute.GroupAttribute) -> list[str]: ...
def GetXmlIOParameterFromAttribute(attr, path, hints, forcePersistant: bool = ...): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import DrawingModule as DrawingModule
import NodegraphAPI as NodegraphAPI
import QT4GLLayerStack as QT4GLLayerStack
import QT4GLLayerStack.LayerStack
import PyQt5.QtCore as QtCore
import Utils as Utils
from UI4.Tabs.NodeGraphTab.Layers.LinkConnectionLayer import LinkConnectionLayer as LinkConnectionLayer
from typing import Set, Tuple

class PortInteractionLayer(QT4GLLayerStack.LayerStack.Layer):
    def __init__(self, *args, **kwargs) -> None: ...
    def _PortInteractionLayer__node_delete_cb(self, eventType, eventID, node: NodegraphAPI.Node, *args, **kwargs): ...
    def _PortInteractionLayer__removeNodeCallback(self, eventType, eventID, node: NodegraphAPI.Node, **kwargs): ...
    def paintGL(self): ...
    def processEvent(self, event): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtCore as QtCore
import Utils as Utils
from typing import ClassVar, Set, Tuple

HIGHEST_PRIORITY: int
NORMAL_PRIORITY: int

class NotificationRecord(PyQt5.QtCore.QObject):
    action: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    priorityChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    recordDeleted: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, title, text) -> None: ...
    def addAction(self, actionName, actionText): ...
    def delete(self): ...
    def getActions(self): ...
    def getPriority(self, observeIgnored: bool = ...): ...
    def getText(self): ...
    def getTimeStamp(self): ...
    def getTitle(self): ...
    def isIgnored(self): ...
    def setIgnored(self, state): ...
    def setPriority(self, priority): ...
    def setText(self, text): ...
    def setTimeStamp(self, stamp): ...
    def setTitle(self, title): ...
    def triggerAction(self, actionName): ...

def AddRecord(record): ...
def CountRecords(): ...
def GetHighestPriorityOfRecords(): ...
def GetRecords(): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import AssetAPI as AssetAPI
import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import PyFnGeolibProducers as FnGeolibProducers
import LookFileBakeAPI.LookFileUtil as LookFileUtil
import LookFileBakeAPI.OutputFormatAPI as OutputFormatAPI
import PyFnGeolib
import PyFnGeolibProducers
import typing
from LookFileBakeAPI.Exceptions import LookFileBakeException as LookFileBakeException
from LookFileBakeAPI.Utils import CheckRootLocations as CheckRootLocations, FindSharedLodGroupLocations as FindSharedLodGroupLocations, GetLookFileProducerRootId as GetLookFileProducerRootId, GetProducerFromOp as GetProducerFromOp
from _typeshed import Incomplete
from typing import Set, Tuple

class BakePrePostHandlerBase:
    def notify(self, assetId: str, rootLocationProducers: dict[str, PyFnGeolibProducers.GeometryProducer], progressCallback: typing.Callable | None, abortCallback: typing.Callable | None): ...

class LookFileBaker:
    additionalSettings: Incomplete
    includeGlobalAttributes: Incomplete
    includeLodInfo: Incomplete
    materialTreeRootLocations: Incomplete
    outputFormatName: Incomplete
    progressCallback: Incomplete
    sourceAsset: Incomplete
    sourceFile: Incomplete
    traversalObserver: Incomplete
    def __init__(self, outputFormatName: str) -> None: ...
    @staticmethod
    def _appendCelMatchOp(op, txn): ...
    @staticmethod
    def _appendLightsDefaultValuesBackupOp(op, txn, rootLocations): ...
    @staticmethod
    def _getModifiedPassProducer(op, rootLocations): ...
    def bake(self, referenceOp: PyFnGeolib.GeolibRuntimeOp, passNamesAndOps, rootLocations: typing.Iterable[str], outputPath) -> list[str]: ...
    def bakeAndPublish(self, referenceOp: PyFnGeolib.GeolibRuntimeOp, passNamesAndOps, rootLocations: typing.Iterable[str], assetId: str, assetArgs: Incomplete | None = ..., preBakeHandler: Incomplete | None = ..., postBakeHandler: Incomplete | None = ...) -> str: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyQt5.QtCore
import PyQt5.QtGui
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
import Utils as Utils
from UI4.Tabs.BaseTab import BaseTab as BaseTab
from typing import ClassVar, Set, Tuple

class HistoryItem(PyQt5.QtWidgets.QTreeWidgetItem):
    def __init__(self, *args) -> None: ...
    def buildContextMenu(self, menu): ...
    def doRestore(self): ...
    def isAlreadyRestored(self): ...
    def paintCell(self, painter, colors, column, width, align): ...
    def paintFocus(self, painter, colorGroup, rect): ...
    def setColorGroup(self, colorGroup): ...

class UndoHistoryPanel(BaseTab):
    def __init__(self, parent) -> None: ...
    def _UndoHistoryPanel__clearHistory_CB(self): ...
    def _UndoHistoryPanel__contextMenu_CB(self, point): ...
    def _UndoHistoryPanel__freeze(self): ...
    def _UndoHistoryPanel__restoreToSelected_CB(self): ...
    def _UndoHistoryPanel__selectionChanged_CB(self, item): ...
    def _UndoHistoryPanel__thaw(self): ...
    def _UndoHistoryPanel__undoCallback(self): ...
    def hideEvent(self, event: PyQt5.QtGui.QHideEvent): ...
    def showEvent(self, event: PyQt5.QtGui.QShowEvent): ...
    def sync(self): ...

class UndoHistoryPanelTreeWidget(PyQt5.QtWidgets.QTreeWidget):
    selectionChangedSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, parent) -> None: ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import QT4Color.ColorTransforms as ColorTransforms
import QT4Color.Degenerate as Degenerate
import QT4Color.Globals as Globals
import PyOpenColorIO as OCIO
import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets
from typing import ClassVar, Set, Tuple

class Gradient2DWidget(PyQt5.QtWidgets.QFrame):
    valueChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, color, uMode, vMode, parent, enableFilmlook: bool = ..., enableNoFilmlookColorSpace: bool = ...) -> None: ...
    def _Gradient2DWidget__gradientDestroyedCB(self): ...
    def _Gradient2DWidget__updateGradient(self): ...
    def _Gradient2DWidget__wheelEventCB(self, delta): ...
    def setColor(self, color): ...
    def setFilmlookEnabled(self, enabled): ...
    def setMode(self, uMode, vMode): ...
    def setNoFilmlookColorSpaceEnabled(self, enabled): ...
    def setReadOnly(self, value): ...
    def sizeHint(self): ...
    def sizePolicy(self): ...

class GradientDrawable(PyQt5.QtWidgets.QOpenGLWidget):
    CURSOR_CIRCLE: ClassVar[None] = ...
    valueChanged: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    wheelEventSignal: ClassVar[PyQt5.QtCore.pyqtSignal] = ...
    def __init__(self, *args) -> None: ...
    def _GradientDrawable__drawGradient(self): ...
    def _GradientDrawable__drawMark(self): ...
    def _GradientDrawable__posToRgb(self, position): ...
    def _GradientDrawable__sendValueChangedFromMouse(self, ev, final: bool = ...): ...
    def _GradientDrawable__windowPosToUV(self, pos): ...
    def getPickingColorspace(self): ...
    def mouseMoveEvent(self, ev): ...
    def mousePressEvent(self, ev): ...
    def mouseReleaseEvent(self, ev): ...
    def paintGL(self): ...
    def resizeGL(self, width, height): ...
    def setColorAndModes(self, rgb, uMode, vMode): ...
    def setFilmlookEnabled(self, enabled): ...
    def setNoFilmlookColorSpaceEnabled(self, enabled): ...
    def setReadOnly(self, value): ...
    def wheelEvent(self, ev): ...



# mypy: disable-error-code="misc, override, attr-defined, no-redef, assignment"

import PyFnAttribute as FnAttribute
import PyFnGeolib as FnGeolib
import Nodes3DAPI.Node3D as Node3D
import NodegraphAPI as NodegraphAPI
from Nodes3DAPI.GenericAssign import GenericAssign as GenericAssign
from typing import ClassVar, Set, Tuple

_ExtraHints: dict
_Parameter_XML: str

class LookFileGlobalsAssignBaseType3D(GenericAssign):
    __pychecker__: ClassVar[str] = ...
    def __init__(self) -> None: ...
    def _filterAttrList(self, graphState: NodegraphAPI.GraphState, attrList): ...
    def _getOpChain(self, interface): ...
    def addParameterHints(self, attrName, inputDict): ...

def GraphStateInLookfileBake(graphState: NodegraphAPI.GraphState): ...
